"""cartaporte20 http://www.sat.gob.mx/CartaPorte20"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class Domicilio(ScalarMap):
    """
    Nodo opcional para registrar información del domicilio del(los) tipo(s) de figura transporte que intervenga(n) en el traslado de los bienes y/o mercancías.
    
    :param estado: Atributo requerido para registrar el estado, entidad, región, comunidad, o dato análogo en donde se encuentra ubicado el domicilio del(los) tipo(s) de figura transporte.
    :param pais: Atributo requerido que sirve para registrar la clave del país en donde se encuentra ubicado el domicilio del(los) tipo(s) de figura transporte, conforme al catálogo c_Pais del CFDI publicado en el portal del SAT en Internet de acuerdo a la especificación ISO 3166-1.
    :param codigo_postal: Atributo requerido para registrar el código postal en donde se encuentra ubicado el domicilio del(los) tipo(s) de figura transporte.
    :param calle: Atributo opcional que sirve para registrar la calle en la que está ubicado el domicilio del(los) tipo(s) de figura transporte.
    :param numero_exterior: Atributo opcional que sirve para registrar el número exterior en donde se ubica el domicilio del(los) tipo(s) de figura transporte.
    :param numero_interior: Atributo opcional que sirve para registrar el número interior, en caso de existir, en donde se ubica el domicilio del(los) tipo(s) de figura transporte.
    :param colonia: Atributo opcional que sirve para expresar la clave de la colonia o dato análogo en donde se ubica el domicilio del(los) tipo(s) de figura transporte.
    :param localidad: Atributo opcional para registrar la clave de la ciudad, población, distrito o dato análogo de donde se encuentra ubicado el domicilio del(los) tipo(s) de figura transporte.
    :param referencia: Atributo opcional para registrar una referencia geográfica adicional que permita una fácil o precisa ubicación del domicilio del(los) tipo(s) de figura transporte; por ejemplo, las coordenadas del GPS.
    :param municipio: Atributo opcional para registrar la clave del municipio, delegación o alcaldía, condado o dato análogo en donde se encuentra ubicado el domicilio del(los) tipo(s) de figura transporte.
    """
    
    def __init__(
            self,
            estado: str,
            pais: str,
            codigo_postal: str,
            calle: str = None,
            numero_exterior: str = None,
            numero_interior: str = None,
            colonia: str = None,
            localidad: str = None,
            referencia: str = None,
            municipio: str = None,
    ): 
        super().__init__({
            'Estado': estado,
            'Pais': pais,
            'CodigoPostal': codigo_postal,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'NumeroInterior': numero_interior,
            'Colonia': colonia,
            'Localidad': localidad,
            'Referencia': referencia,
            'Municipio': municipio,
        })
        

class TiposFigura(ScalarMap):
    """
    Nodo condicional para indicar los datos del(los) tipo(s) de figura(s) que participan en el traslado de los bienes y/o mercancías en los distintos medios de transporte.
    
    :param tipo_figura: Atributo requerido para registrar la clave de la figura de transporte que interviene en el traslado de los bienes y/o mercancías.
    :param rfc_figura: Atributo condicional para registrar el RFC de la figura de transporte que interviene en el traslado de los bienes y/o mercancías.
    :param num_licencia: Atributo condicional para expresar el número de la licencia o el permiso otorgado al operador del autotransporte de carga en el que realiza el traslado de los bienes y/o mercancías.
    :param nombre_figura: Atributo opcional para registrar el nombre de la figura de transporte que interviene en el traslado de los bienes y/o mercancías.
    :param num_reg_id_trib_figura: Atributo condicional para registrar el número de identificación o registro fiscal del país de residencia de la figura de transporte que interviene en el traslado de los bienes y/o mercancías, cuando se trate de residentes en el extranjero para los efectos fiscales correspondientes.
    :param residencia_fiscal_figura: Atributo condicional para registrar la clave del país de residencia de la figura de transporte que interviene en el traslado de los bienes y/o mercancías para los efectos fiscales correspondientes.
    :param partes_transporte: Nodo condicional para indicar los datos de las partes del transporte de las cuales el emisor del comprobante es distinto al dueño de las mismas, por ejemplo: vehículos, máquinas, contenedores, plataformas, etc; mismos que son utilizados para el traslado de los bienes y/o mercancías.
    :param domicilio: Nodo opcional para registrar información del domicilio del(los) tipo(s) de figura transporte que intervenga(n) en el traslado de los bienes y/o mercancías.
    """
    
    def __init__(
            self,
            tipo_figura: str,
            rfc_figura: str = None,
            num_licencia: str = None,
            nombre_figura: str = None,
            num_reg_id_trib_figura: str = None,
            residencia_fiscal_figura: str = None,
            partes_transporte: str | Sequence[str] = None,
            domicilio: Domicilio | dict = None,
    ): 
        super().__init__({
            'TipoFigura': tipo_figura,
            'RFCFigura': rfc_figura,
            'NumLicencia': num_licencia,
            'NombreFigura': nombre_figura,
            'NumRegIdTribFigura': num_reg_id_trib_figura,
            'ResidenciaFiscalFigura': residencia_fiscal_figura,
            'PartesTransporte': partes_transporte,
            'Domicilio': domicilio,
        })
        

class DerechosDePaso(ScalarMap):
    """
    Nodo opcional para registrar los tipos de derechos de paso cubiertos por el transportista en las vías férreas de las cuales no es concesionario o asignatario, así como la distancia establecida en kilómetros.
    
    :param tipo_derecho_de_paso: Atributo requerido para registrar la clave del derecho de paso pagado por el transportista en las vías férreas de las cuales no es concesionario o asignatario.
    :param kilometraje_pagado: Atributo requerido para registrar el total de kilómetros pagados por el transportista en las vías férreas de las cuales no es concesionario o asignatario con el derecho de paso.
    """
    
    def __init__(
            self,
            tipo_derecho_de_paso: str,
            kilometraje_pagado: Decimal | int,
    ): 
        super().__init__({
            'TipoDerechoDePaso': tipo_derecho_de_paso,
            'KilometrajePagado': kilometraje_pagado,
        })
        

class Contenedor(ScalarMap):
    """
    Nodo condicional para especificar el tipo de contenedor o vagón en el que se trasladan los bienes y/o mercancías por vía férrea.
    
    :param tipo_contenedor: Atributo requerido para registrar la clave con la que se identifica al tipo de contenedor o el vagón en el que se realiza el traslado de los bienes y/o mercancías.
    :param peso_contenedor_vacio: Atributo requerido para registrar en kilogramos, el peso del contenedor vacío en el que se trasladan los bienes y/o mercancías.
    :param peso_neto_mercancia: Atributo requerido para registrar en kilogramos el peso neto de los bienes y/o mercancías que son trasladados en el contenedor.
    """
    
    def __init__(
            self,
            tipo_contenedor: str,
            peso_contenedor_vacio: Decimal | int,
            peso_neto_mercancia: Decimal | int,
    ): 
        super().__init__({
            'TipoContenedor': tipo_contenedor,
            'PesoContenedorVacio': peso_contenedor_vacio,
            'PesoNetoMercancia': peso_neto_mercancia,
        })
        

class Carro(ScalarMap):
    """
    Nodo requerido para registrar la información que permite identificar el (los) carro(s) en el (los) que se trasladan los bienes y/o mercancías por vía férrea.
    
    :param tipo_carro: Atributo requerido para registrar la clave del tipo de carro utilizado para el traslado de los bienes y/o mercancías por vía férrea.
    :param matricula_carro: Atributo requerido para registrar el número de contenedor, carro de ferrocarril o número económico del vehículo en el que se trasladan los bienes y/o mercancías por vía férrea.
    :param guia_carro: Atributo requerido para registrar el número de guía asignado al contenedor, carro de ferrocarril o vehículo, en el que se trasladan los bienes y/o mercancías por vía férrea.
    :param toneladas_netas_carro: Atributo requerido para registrar la cantidad de las toneladas netas depositadas en el contenedor, carro de ferrocarril o vehículo en el que se trasladan los bienes y/o mercancías por vía férrea.
    :param contenedor: Nodo condicional para especificar el tipo de contenedor o vagón en el que se trasladan los bienes y/o mercancías por vía férrea.
    """
    
    def __init__(
            self,
            tipo_carro: str,
            matricula_carro: str,
            guia_carro: str,
            toneladas_netas_carro: Decimal | int,
            contenedor: Contenedor | dict | Sequence[Contenedor | dict] = None,
    ): 
        super().__init__({
            'TipoCarro': tipo_carro,
            'MatriculaCarro': matricula_carro,
            'GuiaCarro': guia_carro,
            'ToneladasNetasCarro': toneladas_netas_carro,
            'Contenedor': contenedor,
        })
        

class TransporteFerroviario(ScalarMap):
    """
    Nodo condicional para registrar la información que permita la identificación del carro o contenedor en el que se trasladan los bienes y/o mercancías por vía férrea.
    
    :param tipo_de_servicio: Atributo requerido para registrar la clave del tipo de servicio utilizado para el traslado de los bienes y/o mercancías por vía férrea.
    :param tipo_de_trafico: Atributo requerido para registrar la clave del tipo de tráfico (interrelación entre concesionarios) para realizar el traslado de los bienes y/o mercancías por vía férrea dentro del territorio nacional.
    :param carro: Nodo requerido para registrar la información que permite identificar el (los) carro(s) en el (los) que se trasladan los bienes y/o mercancías por vía férrea.
    :param nombre_aseg: Atributo opcional para registrar el nombre de la aseguradora que cubre los riesgos para el traslado de los bienes y/o mercancías por vía férrea.
    :param num_poliza_seguro: Atributo opcional para registrar el número de póliza asignada por la aseguradora para la protección e indemnización por responsabilidad civil en el traslado de los bienes y/o mercancías que se realiza por vía férrea.
    :param derechos_de_paso: Nodo opcional para registrar los tipos de derechos de paso cubiertos por el transportista en las vías férreas de las cuales no es concesionario o asignatario, así como la distancia establecida en kilómetros.
    """
    
    def __init__(
            self,
            tipo_de_servicio: str,
            tipo_de_trafico: str,
            carro: Carro | dict | Sequence[Carro | dict],
            nombre_aseg: str = None,
            num_poliza_seguro: str = None,
            derechos_de_paso: DerechosDePaso | dict | Sequence[DerechosDePaso | dict] = None,
    ): 
        super().__init__({
            'TipoDeServicio': tipo_de_servicio,
            'TipoDeTrafico': tipo_de_trafico,
            'Carro': carro,
            'NombreAseg': nombre_aseg,
            'NumPolizaSeguro': num_poliza_seguro,
            'DerechosDePaso': derechos_de_paso,
        })
        

class TransporteAereo(ScalarMap):
    """
    Nodo condicional para registrar la información que permita la identificación del transporte aéreo por medio del cual se trasladan los bienes y/o mercancías.
    
    :param perm_sct: Atributo requerido para registrar la clave del permiso proporcionado por la SCT o la autoridad análoga, la cual debe corresponder con la aeronave que se está utilizando para realizar el traslado de los bienes y/o mercancías por vía aérea.
    :param num_permiso_sct: Atributo requerido para registrar el número de permiso o valor análogo proporcionado por la SCT o la autoridad análoga, según corresponda, para el transporte de bienes y/o mercancías por vía aérea.
    :param numero_guia: Atributo requerido para registrar el número de guía aérea con el que se trasladan los bienes y/o mercancías.
    :param codigo_transportista: Atributo requerido para registrar el valor del código que tiene asignado el transportista el cual debe contener alguna de las claves contenidas en el catálogo correspondiente.
    :param matricula_aeronave: Atributo opcional para registrar el número de la matrícula de la aeronave con la que se realiza el traslado de los bienes y/o mercancías en territorio nacional el cual tiene una longitud de 10 posiciones y se compone de valores alfanuméricos, más el carácter especial denominado guion medio “-“.
    :param nombre_aseg: Atributo opcional para registrar el nombre de la aseguradora que cubre los riesgos de la aeronave con la que transportan los bienes y/o mercancías.
    :param num_poliza_seguro: Atributo opcional para registrar el número de póliza asignado por la aseguradora que cubre la protección e indemnización por responsabilidad civil de la aeronave que transporta los bienes y/o mercancías.
    :param lugar_contrato: Atributo opcional para registrar el lugar, entidad, región, localidad o análogo, donde se celebró el contrato para realizar el traslado de los bienes y/o mercancías.
    :param rfc_embarcador: Atributo opcional para registrar el RFC del embarcador de los bienes y/o mercancías que se trasladan.
    :param num_reg_id_trib_embarc: Atributo condicional para incorporar el número de identificación o registro fiscal del país de residencia cuando el embarcador sea residente en el extranjero para los efectos fiscales correspondientes de los bienes y/o mercancías que se trasladan.
    :param residencia_fiscal_embarc: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del embarcador de los bienes y/o mercancías.
    :param nombre_embarcador: Atributo opcional para registrar el nombre del embarcador de los bienes y/o mercancías que se trasladan, ya sea nacional o extranjero.
    """
    
    def __init__(
            self,
            perm_sct: str,
            num_permiso_sct: str,
            numero_guia: str,
            codigo_transportista: str,
            matricula_aeronave: str = None,
            nombre_aseg: str = None,
            num_poliza_seguro: str = None,
            lugar_contrato: str = None,
            rfc_embarcador: str = None,
            num_reg_id_trib_embarc: str = None,
            residencia_fiscal_embarc: str = None,
            nombre_embarcador: str = None,
    ): 
        super().__init__({
            'PermSCT': perm_sct,
            'NumPermisoSCT': num_permiso_sct,
            'NumeroGuia': numero_guia,
            'CodigoTransportista': codigo_transportista,
            'MatriculaAeronave': matricula_aeronave,
            'NombreAseg': nombre_aseg,
            'NumPolizaSeguro': num_poliza_seguro,
            'LugarContrato': lugar_contrato,
            'RFCEmbarcador': rfc_embarcador,
            'NumRegIdTribEmbarc': num_reg_id_trib_embarc,
            'ResidenciaFiscalEmbarc': residencia_fiscal_embarc,
            'NombreEmbarcador': nombre_embarcador,
        })
        

class TransporteMaritimo(ScalarMap):
    """
    Nodo condicional para registrar la información que permita la identificación de la embarcación a través de la cual se trasladan los bienes y/o mercancías por vía marítima.
    
    :param tipo_embarcacion: Atributo requerido para registrar la clave de identificación del tipo de embarcación que es utilizado para trasladar los bienes y/o mercancías.
    :param matricula: Atributo requerido para registrar el número de la matrícula o registro de la embarcación que es utilizada para transportar los bienes y/o mercancías.
    :param numero_omi: Atributo requerido para registrar el número de identificación asignado por la Organización Marítima Internacional, a la embarcación encargada de transportar los bienes y/o mercancías.
    :param nacionalidad_embarc: Atributo requerido para registrar la clave del país correspondiente a la nacionalidad de la embarcación que transporta los bienes y/o mercancías.
    :param unidades_de_arq_bruto: Atributo requerido para registrar el valor de las unidades de arqueo bruto conforme a las medidas internacionales definidas por el ITC para cada tipo de buque o embarcación en la que se transportan los bienes y/o mercancías.
    :param tipo_carga: Atributo requerido para especificar el tipo de carga en el cual se clasifican los bienes y/o mercancías que se transportan en la embarcación.
    :param num_cert_itc: Atributo requerido para registrar el número del certificado emitido por la ITC para la embarcación o buque que transporta los bienes y/o mercancías.
    :param nombre_agente_naviero: Atributo requerido para registrar el nombre del agente naviero consignatario autorizado para gestionar el traslado de los bienes y/o mercancías por vía marítima.
    :param num_autorizacion_naviero: Atributo requerido para expresar el número de la autorización como agente naviero consignatario emitida por la SCT.
    :param contenedor: Nodo requerido para registrar los datos del contenedor en el que se transportan los bienes y/o mercancías.
    :param perm_sct: Atributo opcional para registrar la clave del permiso proporcionado por la SCT, la cual debe corresponder con la embarcación que se está utilizando para el traslado de los bienes y/o mercancías, de acuerdo al catálogo correspondiente.
    :param num_permiso_sct: Atributo opcional para registrar el número del permiso otorgado por la SCT a la embarcación utilizada para el traslado de los bienes y/o mercancías.
    :param nombre_aseg: Atributo opcional para registrar el nombre de la aseguradora que cubre la protección e indemnización por responsabilidad civil de la embarcación en el traslado de los bienes y/o mercancías.
    :param num_poliza_seguro: Atributo opcional para registrar el número de póliza asignada por la aseguradora que cubre la protección e indemnización por responsabilidad civil de la embarcación en el traslado de los bienes y/o mercancías.
    :param anio_embarcacion: Atributo opcional para registrar el año de la embarcación en la que se transportan los bienes y/o mercancías.
    :param nombre_embarc: Atributo opcional para registrar el nombre de la embarcación en la que se realiza el traslado de los bienes y/o mercancías.
    :param eslora: Atributo opcional para registrar la longitud de eslora, definida en pies, con la que cuenta la embarcación o el buque en el que se transportan los bienes y/o mercancías.
    :param manga: Atributo opcional para registrar la longitud de manga, definida en pies, con la que cuenta la embarcación o el buque en el que se transportan los bienes y/o mercancías.
    :param calado: Atributo opcional para registrar la longitud del calado, definida en pies, con la que cuenta la embarcación o el buque en el que se transportan los bienes y/o mercancías.
    :param linea_naviera: Atributo opcional para registrar el nombre de la línea naviera autorizada de gestionar el traslado de los bienes y/o mercancías por vía marítima.
    :param num_viaje: Atributo opcional para registrar el número del viaje con el que se identifica el traslado de los bienes y/o mercancías en el buque o la embarcación.
    :param num_conoc_embarc: Atributo opcional para registrar el número de conocimiento de embarque con el que se identifica el traslado de los bienes y/o mercancías.
    """
    
    def __init__(
            self,
            tipo_embarcacion: str,
            matricula: str,
            numero_omi: str,
            nacionalidad_embarc: str,
            unidades_de_arq_bruto: Decimal | int,
            tipo_carga: str,
            num_cert_itc: str,
            nombre_agente_naviero: str,
            num_autorizacion_naviero: str,
            contenedor: Contenedor | dict | Sequence[Contenedor | dict],
            perm_sct: str = None,
            num_permiso_sct: str = None,
            nombre_aseg: str = None,
            num_poliza_seguro: str = None,
            anio_embarcacion: int = None,
            nombre_embarc: str = None,
            eslora: Decimal | int = None,
            manga: Decimal | int = None,
            calado: Decimal | int = None,
            linea_naviera: str = None,
            num_viaje: str = None,
            num_conoc_embarc: str = None,
    ): 
        super().__init__({
            'TipoEmbarcacion': tipo_embarcacion,
            'Matricula': matricula,
            'NumeroOMI': numero_omi,
            'NacionalidadEmbarc': nacionalidad_embarc,
            'UnidadesDeArqBruto': unidades_de_arq_bruto,
            'TipoCarga': tipo_carga,
            'NumCertITC': num_cert_itc,
            'NombreAgenteNaviero': nombre_agente_naviero,
            'NumAutorizacionNaviero': num_autorizacion_naviero,
            'Contenedor': contenedor,
            'PermSCT': perm_sct,
            'NumPermisoSCT': num_permiso_sct,
            'NombreAseg': nombre_aseg,
            'NumPolizaSeguro': num_poliza_seguro,
            'AnioEmbarcacion': anio_embarcacion,
            'NombreEmbarc': nombre_embarc,
            'Eslora': eslora,
            'Manga': manga,
            'Calado': calado,
            'LineaNaviera': linea_naviera,
            'NumViaje': num_viaje,
            'NumConocEmbarc': num_conoc_embarc,
        })
        

class Remolque(ScalarMap):
    """
    Nodo requerido para expresar la información del(los) remolque(s) o semirremolque(s) que se adapta(n) al autotransporte para realizar el traslado de los bienes y/o mercancías.
    
    :param sub_tipo_rem: Atributo requerido para expresar la clave del subtipo de remolque o semirremolques que se emplean con el autotransporte para el traslado de los bienes y/o mercancías.
    :param placa: Atributo requerido para registrar los caracteres alfanuméricos, sin guiones ni espacios de la placa vehicular del remolque o semirremolque que es utilizado para transportar los bienes y/o mercancías.
    """
    
    def __init__(
            self,
            sub_tipo_rem: str,
            placa: str,
    ): 
        super().__init__({
            'SubTipoRem': sub_tipo_rem,
            'Placa': placa,
        })
        

class Seguros(ScalarMap):
    """
    Nodo requerido para registrar los datos de las pólizas de seguro que cubren los riesgos en el traslado de los bienes y/o mercancías.
    
    :param asegura_resp_civil: Atributo requerido para registrar el nombre de la aseguradora que cubre los riesgos por responsabilidad civil del autotransporte utilizado para el traslado de los bienes y/o mercancías.
    :param poliza_resp_civil: Atributo requerido para registrar el número de póliza asignado por la aseguradora, que cubre los riesgos por responsabilidad civil del autotransporte utilizado para el traslado de los bienes y/o mercancías.
    :param asegura_med_ambiente: Atributo condicional para registrar el nombre de la aseguradora, que cubre los posibles daños al medio ambiente (aplicable para los transportistas de materiales, residuos o remanentes y desechos peligrosos).
    :param poliza_med_ambiente: Atributo condicional para registrar el número de póliza asignado por la aseguradora, que cubre los posibles daños al medio ambiente (aplicable para los transportistas de materiales, residuos o remanentes y desechos peligrosos).
    :param asegura_carga: Atributo opcional para registrar el nombre de la aseguradora que cubre los riesgos de la carga (bienes y/o mercancías) del autotransporte utilizado para el traslado.
    :param poliza_carga: Atributo opcional para expresar el número de póliza asignado por la aseguradora que cubre los riesgos de la carga (bienes y/o mercancías) del autotransporte utilizado para el traslado.
    :param prima_seguro: Atributo opcional para registrar el valor del importe por el cargo adicional convenido entre el transportista y el cliente, el cual será igual al valor de la prima del seguro contratado, conforme a lo establecido en la cláusula novena del Acuerdo por el que se homologa la Carta de Porte regulada por la Ley de Caminos, Puentes y Autotransporte Federal, con el complemento Carta Porte que debe acompañar al Comprobante Fiscal Digital por Internet (CFDI).
    """
    
    def __init__(
            self,
            asegura_resp_civil: str,
            poliza_resp_civil: str,
            asegura_med_ambiente: str = None,
            poliza_med_ambiente: str = None,
            asegura_carga: str = None,
            poliza_carga: str = None,
            prima_seguro: Decimal | int = None,
    ): 
        super().__init__({
            'AseguraRespCivil': asegura_resp_civil,
            'PolizaRespCivil': poliza_resp_civil,
            'AseguraMedAmbiente': asegura_med_ambiente,
            'PolizaMedAmbiente': poliza_med_ambiente,
            'AseguraCarga': asegura_carga,
            'PolizaCarga': poliza_carga,
            'PrimaSeguro': prima_seguro,
        })
        

class IdentificacionVehicular(ScalarMap):
    """
    Nodo requerido para registrar los datos de identificación del autotransporte en el que se trasladan los bienes y/o mercancías.
    
    :param config_vehicular: Atributo requerido para expresar la clave de nomenclatura del autotransporte que es utilizado para transportar los bienes y/o mercancías.
    :param placa_vm: Atributo requerido para registrar solo los caracteres alfanuméricos, sin guiones ni espacios de la placa vehicular del autotransporte que es utilizado para transportar los bienes y/o mercancías.
    :param anio_modelo_vm: Atributo requerido para registrar el año del autotransporte que es utilizado para transportar los bienes y/o mercancías.
    """
    
    def __init__(
            self,
            config_vehicular: str,
            placa_vm: str,
            anio_modelo_vm: int,
    ): 
        super().__init__({
            'ConfigVehicular': config_vehicular,
            'PlacaVM': placa_vm,
            'AnioModeloVM': anio_modelo_vm,
        })
        

class Autotransporte(ScalarMap):
    """
    Nodo condicional para registrar la información que permita la identificación del autotransporte de carga, por medio del cual se trasladan los bienes y/o mercancías, que transitan a través de las carreteras del territorio nacional.
    
    :param perm_sct: Atributo requerido para registrar la clave del tipo de permiso proporcionado por la SCT o la autoridad análoga, el cual debe corresponder con el tipo de autotransporte utilizado para el traslado de los bienes y/o mercancías de acuerdo al catálogo correspondiente.
    :param num_permiso_sct: Atributo requerido para registrar el número del permiso otorgado por la SCT o la autoridad correspondiente, al autotransporte utilizado para el traslado de los bienes y/o mercancías.
    :param identificacion_vehicular: Nodo requerido para registrar los datos de identificación del autotransporte en el que se trasladan los bienes y/o mercancías.
    :param seguros: Nodo requerido para registrar los datos de las pólizas de seguro que cubren los riesgos en el traslado de los bienes y/o mercancías.
    :param remolques: Nodo condicional para registrar los datos del(los) remolque(s) o semirremolque(s) que se adaptan al autotransporte para realizar el traslado de los bienes y/o mercancías.
    """
    
    def __init__(
            self,
            perm_sct: str,
            num_permiso_sct: str,
            identificacion_vehicular: IdentificacionVehicular | dict,
            seguros: Seguros | dict,
            remolques: Remolque | dict | Sequence[Remolque | dict] = None,
    ): 
        super().__init__({
            'PermSCT': perm_sct,
            'NumPermisoSCT': num_permiso_sct,
            'IdentificacionVehicular': identificacion_vehicular,
            'Seguros': seguros,
            'Remolques': remolques,
        })
        

class DetalleMercancia(ScalarMap):
    """
    Nodo condicional para registrar especificaciones de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    
    :param unidad_peso_merc: Atributo requerido para registrar la clave de la unidad de medida estandarizada del peso de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param peso_bruto: Atributo requerido para registrar el peso bruto total de los bienes y/o mercancías que se trasladan a través de los diferentes medios de transporte.
    :param peso_neto: Atributo requerido para registrar el peso neto total de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param peso_tara: Atributo requerido para registrar el peso bruto, menos el peso neto de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    :param num_piezas: Atributo opcional para registrar el número de piezas de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    """
    
    def __init__(
            self,
            unidad_peso_merc: str,
            peso_bruto: Decimal | int,
            peso_neto: Decimal | int,
            peso_tara: Decimal | int,
            num_piezas: int = None,
    ): 
        super().__init__({
            'UnidadPesoMerc': unidad_peso_merc,
            'PesoBruto': peso_bruto,
            'PesoNeto': peso_neto,
            'PesoTara': peso_tara,
            'NumPiezas': num_piezas,
        })
        

class CantidadTransporta(ScalarMap):
    """
    Nodo opcional para registrar la cantidad de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte, que será captada o distribuida en distintos puntos, a fin de identificar el punto de origen y destino correspondiente.
    
    :param cantidad: Atributo requerido para expresar el número de bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param id_origen: Atributo requerido para expresar la clave del identificador del origen de los bienes y/o mercancías que se trasladan por los distintos medios de transporte, de acuerdo al valor registrado en el atributo “IDUbicacion”, del nodo “Ubicacion”.
    :param id_destino: Atributo requerido para registrar la clave del identificador del destino de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte, de acuerdo al valor registrado en el atributo “IDUbicacion”, del nodo “Ubicacion”.
    :param cves_transporte: Atributo condicional para indicar la clave a través de la cual se identifica el medio por el que se transportan los bienes y/o mercancías.
    """
    
    def __init__(
            self,
            cantidad: Decimal | int,
            id_origen: str,
            id_destino: str,
            cves_transporte: str = None,
    ): 
        super().__init__({
            'Cantidad': cantidad,
            'IDOrigen': id_origen,
            'IDDestino': id_destino,
            'CvesTransporte': cves_transporte,
        })
        

class GuiasIdentificacion(ScalarMap):
    """
    Nodo condicional para registrar la información del(los) número(s) de guía(s) que se encuentre(n) asociado(s) al(los) paquete(s) que se traslada(n) dentro del territorio nacional.
    
    :param numero_guia_identificacion: Atributo requerido para expresar el número de guía de cada paquete que se encuentra asociado con el traslado de los bienes y/o mercancías en territorio nacional.
    :param descrip_guia_identificacion: Atributo requerido para expresar la descripción del contenido del paquete o carga registrada en la guía, o en el número de identificación, que se encuentra asociado con el traslado de los bienes y/o mercancías dentro del territorio nacional.
    :param peso_guia_identificacion: Atributo requerido para indicar en kilogramos, el peso del paquete o carga que se está trasladando en territorio nacional y que se encuentra registrado en la guía o el número de identificación correspondiente.
    """
    
    def __init__(
            self,
            numero_guia_identificacion: str,
            descrip_guia_identificacion: str,
            peso_guia_identificacion: Decimal | int,
    ): 
        super().__init__({
            'NumeroGuiaIdentificacion': numero_guia_identificacion,
            'DescripGuiaIdentificacion': descrip_guia_identificacion,
            'PesoGuiaIdentificacion': peso_guia_identificacion,
        })
        

class Mercancia(ScalarMap):
    """
    Nodo requerido para registrar detalladamente la información de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    
    :param bienes_transp: Atributo requerido para registrar la clave de producto de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param descripcion: Atributo requerido para detallar las características de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param cantidad: Atributo requerido para expresar la cantidad total de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    :param clave_unidad: Atributo requerido para registrar la clave de la unidad de medida estandarizada aplicable para la cantidad de los bienes y/o mercancías que se trasladan en los distintos medios de transporte. La unidad debe corresponder con la descripción de los bienes y/o mercancías registrados.
    :param peso_en_kg: Atributo requerido para indicar en kilogramos el peso estimado de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param clave_stcc: Atributo opcional para expresar la clave de producto de la STCC (por sus siglas en inglés, Standard Transportation Commodity Code), cuando el medio de transporte utilizado para el traslado de los bienes y/o mercancías sea ferroviario.
    :param unidad: Atributo opcional para registrar la unidad de medida propia para la cantidad de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte. La unidad debe corresponder con la descripción de los bienes y/o mercancías.
    :param dimensiones: Atributo opcional para expresar las medidas del empaque de los bienes y/o mercancías que se trasladan en los distintos medios de transporte. Se debe registrar la longitud, la altura y la anchura en centímetros o en pulgadas, separados dichos valores con una diagonal, i.e. 30/40/30cm.
    :param material_peligroso: Atributo condicional para precisar que los bienes y/o mercancías que se trasladan son considerados o clasificados como material peligroso.
    :param cve_material_peligroso: Atributo condicional para indicar la clave del tipo de material peligroso que se transporta de acuerdo a la NOM-002-SCT/2011.
    :param embalaje: Atributo condicional para precisar la clave del tipo de embalaje que se requiere para transportar el material o residuo peligroso.
    :param descrip_embalaje: Atributo opcional para expresar la descripción del embalaje de los bienes y/o mercancías que se trasladan y que se consideran material o residuo peligroso.
    :param valor_mercancia: Atributo condicional para expresar el monto del valor de los bienes y/o mercancías que se trasladan en los distintos medios de transporte, de acuerdo al valor mercado, al valor pactado en la contraprestación o bien al valor estimado que determine el contribuyente.
    :param moneda: Atributo condicional para identificar la clave de la moneda utilizada para expresar el valor de los bienes y/o mercancías que se trasladan en los distintos medios de transporte. Cuando se usa moneda nacional se registra MXN, de acuerdo a la especificación ISO 4217.
    :param fraccion_arancelaria: Atributo condicional que sirve para expresar la clave de la fracción arancelaria que corresponde con la descripción de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param uuid_comercio_ext: Atributo opcional para expresar el folio fiscal (UUID) del comprobante de comercio exterior que se relaciona.
    :param pedimentos: Nodo condicional para registrar la información del(los) número(s) de pedimento(s) de importación que se encuentra(n) asociado(s) al traslado de los bienes y/o mercancías de procedencia extranjera para acreditar la legal estancia o tenencia durante su traslado en territorio nacional.
    :param guias_identificacion: Nodo condicional para registrar la información del(los) número(s) de guía(s) que se encuentre(n) asociado(s) al(los) paquete(s) que se traslada(n) dentro del territorio nacional.
    :param cantidad_transporta: Nodo opcional para registrar la cantidad de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte, que será captada o distribuida en distintos puntos, a fin de identificar el punto de origen y destino correspondiente.
    :param detalle_mercancia: Nodo condicional para registrar especificaciones de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    """
    
    def __init__(
            self,
            bienes_transp: str,
            descripcion: str,
            cantidad: Decimal | int,
            clave_unidad: str,
            peso_en_kg: Decimal | int,
            clave_stcc: str = None,
            unidad: str = None,
            dimensiones: str = None,
            material_peligroso: str = None,
            cve_material_peligroso: str = None,
            embalaje: str = None,
            descrip_embalaje: str = None,
            valor_mercancia: Decimal | int = None,
            moneda: str = None,
            fraccion_arancelaria: str = None,
            uuid_comercio_ext: str = None,
            pedimentos: str | Sequence[str] = None,
            guias_identificacion: GuiasIdentificacion | dict | Sequence[GuiasIdentificacion | dict] = None,
            cantidad_transporta: CantidadTransporta | dict | Sequence[CantidadTransporta | dict] = None,
            detalle_mercancia: DetalleMercancia | dict = None,
    ): 
        super().__init__({
            'BienesTransp': bienes_transp,
            'Descripcion': descripcion,
            'Cantidad': cantidad,
            'ClaveUnidad': clave_unidad,
            'PesoEnKg': peso_en_kg,
            'ClaveSTCC': clave_stcc,
            'Unidad': unidad,
            'Dimensiones': dimensiones,
            'MaterialPeligroso': material_peligroso,
            'CveMaterialPeligroso': cve_material_peligroso,
            'Embalaje': embalaje,
            'DescripEmbalaje': descrip_embalaje,
            'ValorMercancia': valor_mercancia,
            'Moneda': moneda,
            'FraccionArancelaria': fraccion_arancelaria,
            'UUIDComercioExt': uuid_comercio_ext,
            'Pedimentos': pedimentos,
            'GuiasIdentificacion': guias_identificacion,
            'CantidadTransporta': cantidad_transporta,
            'DetalleMercancia': detalle_mercancia,
        })
        

class Mercancias(ScalarMap):
    """
    Nodo requerido para registrar la información de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    
    :param peso_bruto_total: Atributo requerido para registrar la suma del peso bruto total estimado de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param unidad_peso: Atributo requerido para registrar la clave de la unidad de medida estandarizada del peso de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    :param num_total_mercancias: Atributo requerido para registrar el número total de los bienes y/o mercancías que se trasladan en los distintos medios de transporte, identificándose por cada nodo "Mercancia" registrado en el complemento.
    :param mercancia: Nodo requerido para registrar detalladamente la información de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param peso_neto_total: Atributo condicional para registrar la suma de los valores indicados en el atributo “PesoNeto” del nodo “DetalleMercancia”.
    :param cargo_por_tasacion: Atributo opcional para expresar el monto del importe pagado por la tasación de los bienes y/o mercancías que se trasladan vía aérea.
    :param autotransporte: Nodo condicional para registrar la información que permita la identificación del autotransporte de carga, por medio del cual se trasladan los bienes y/o mercancías, que transitan a través de las carreteras del territorio nacional.
    :param transporte_maritimo: Nodo condicional para registrar la información que permita la identificación de la embarcación a través de la cual se trasladan los bienes y/o mercancías por vía marítima.
    :param transporte_aereo: Nodo condicional para registrar la información que permita la identificación del transporte aéreo por medio del cual se trasladan los bienes y/o mercancías.
    :param transporte_ferroviario: Nodo condicional para registrar la información que permita la identificación del carro o contenedor en el que se trasladan los bienes y/o mercancías por vía férrea.
    """
    
    def __init__(
            self,
            peso_bruto_total: Decimal | int,
            unidad_peso: str,
            num_total_mercancias: int,
            mercancia: Mercancia | dict | Sequence[Mercancia | dict],
            peso_neto_total: Decimal | int = None,
            cargo_por_tasacion: Decimal | int = None,
            autotransporte: Autotransporte | dict = None,
            transporte_maritimo: TransporteMaritimo | dict = None,
            transporte_aereo: TransporteAereo | dict = None,
            transporte_ferroviario: TransporteFerroviario | dict = None,
    ): 
        super().__init__({
            'PesoBrutoTotal': peso_bruto_total,
            'UnidadPeso': unidad_peso,
            'NumTotalMercancias': num_total_mercancias,
            'Mercancia': mercancia,
            'PesoNetoTotal': peso_neto_total,
            'CargoPorTasacion': cargo_por_tasacion,
            'Autotransporte': autotransporte,
            'TransporteMaritimo': transporte_maritimo,
            'TransporteAereo': transporte_aereo,
            'TransporteFerroviario': transporte_ferroviario,
        })
        

class Ubicacion(ScalarMap):
    """
    Nodo requerido para registrar la ubicación que sirve para indicar el domicilio del origen y/o destino parcial o final, que tienen los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    
    :param tipo_ubicacion: Atributo requerido para precisar si el tipo de ubicación corresponde al origen o destino de las ubicaciones para el traslado de los bienes y/o mercancías en los distintos medios de transporte.
    :param rfc_remitente_destinatario: Atributo requerido para registrar el RFC del remitente o destinatario de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    :param fecha_hora_salida_llegada: Atributo requerido para registrar la fecha y hora estimada en la que salen o llegan los bienes y/o mercancías de origen o al destino, respectivamente. Se expresa en la forma AAAA-MM-DDThh:mm:ss.
    :param id_ubicacion: Atributo condicional para registrar una clave que sirva para identificar el punto de salida o entrada de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte, la cual estará integrada de la siguiente forma: para origen el acrónimo “OR” o para destino el acrónimo “DE” seguido de 6 dígitos numéricos asignados por el contribuyente que emite el comprobante para su identificación.
    :param nombre_remitente_destinatario: Atributo opcional para registrar el nombre del remitente o destinatario de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    :param num_reg_id_trib: Atributo condicional para registrar el número de identificación o registro fiscal del país de residencia, para los efectos fiscales del remitente o destinatario de los bienes y/o mercancías que se trasladan cuando se trate de residentes en el extranjero.
    :param residencia_fiscal: Atributo condicional para registrar la clave del país de residencia para efectos fiscales del remitente o destinatario de los bienes y/o mercancías, conforme el catálogo de CFDI c_Pais publicado en el portal del SAT en Internet de acuerdo a la especificación ISO 3166-1.
    :param num_estacion: Atributo condicional para registrar la clave de la estación de origen o destino para el traslado de los bienes y/o mercancías que se realiza a través de los distintos medios de transporte, esto de acuerdo al valor de la columna “Clave identificación” del catálogo c_Estaciones del complemento Carta Porte que permita asociarla al tipo de transporte.
    :param nombre_estacion: Atributo condicional para registrar el nombre de la estación de origen o destino por la que se pasa para efectuar el traslado de los bienes y/o mercancías a través de los distintos medios de transporte, conforme al catálogo c_Estaciones del complemento Carta Porte.
    :param navegacion_trafico: Atributo condicional para registrar el tipo de puerto de origen o destino en el cual se documentan los bienes y/o mercancías que se trasladan vía marítima.
    :param tipo_estacion: Atributo condicional para registrar el tipo de estación por el que pasan los bienes y/o mercancías durante su traslado a través de los distintos medios de transporte.
    :param distancia_recorrida: Atributo condicional para registrar en kilómetros la distancia recorrida entre la ubicación de origen y la de destino parcial o final, por los distintos medios de transporte que trasladan los bienes y/o mercancías.
    :param domicilio: Nodo condicional para registrar información del domicilio de origen y/o destino de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    """
    
    def __init__(
            self,
            tipo_ubicacion: str,
            rfc_remitente_destinatario: str,
            fecha_hora_salida_llegada: datetime,
            id_ubicacion: str = None,
            nombre_remitente_destinatario: str = None,
            num_reg_id_trib: str = None,
            residencia_fiscal: str = None,
            num_estacion: str = None,
            nombre_estacion: str = None,
            navegacion_trafico: str = None,
            tipo_estacion: str = None,
            distancia_recorrida: Decimal | int = None,
            domicilio: Domicilio | dict = None,
    ): 
        super().__init__({
            'TipoUbicacion': tipo_ubicacion,
            'RFCRemitenteDestinatario': rfc_remitente_destinatario,
            'FechaHoraSalidaLlegada': fecha_hora_salida_llegada,
            'IDUbicacion': id_ubicacion,
            'NombreRemitenteDestinatario': nombre_remitente_destinatario,
            'NumRegIdTrib': num_reg_id_trib,
            'ResidenciaFiscal': residencia_fiscal,
            'NumEstacion': num_estacion,
            'NombreEstacion': nombre_estacion,
            'NavegacionTrafico': navegacion_trafico,
            'TipoEstacion': tipo_estacion,
            'DistanciaRecorrida': distancia_recorrida,
            'Domicilio': domicilio,
        })
        

class CartaPorte(CFDI):
    """
    Complemento para incorporar al Comprobante Fiscal Digital por Internet (CFDI), la información relacionada a los bienes y/o mercancías, ubicaciones de origen, puntos intermedios y destinos, así como lo referente al medio por el que se transportan; ya sea por vía terrestre (autotransporte y férrea), marítima y/o aérea; además de incluir el traslado de hidrocarburos y petrolíferos.
    
    :param transp_internac: Atributo requerido para expresar si los bienes y/o mercancías que son transportadas ingresan o salen del territorio nacional.
    :param ubicaciones: Nodo requerido para registrar las distintas ubicaciones que sirven para indicar el domicilio del origen y/o destino que tienen los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    :param mercancias: Nodo requerido para registrar la información de los bienes y/o mercancías que se trasladan en los distintos medios de transporte.
    :param entrada_salida_merc: Atributo condicional para precisar si los bienes y/o mercancías ingresan o salen del territorio nacional.
    :param pais_origen_destino: Atributo condicional para registrar la clave del país de origen o destino de los bienes y/o mercancías que se trasladan a través de los distintos medios de transporte.
    :param via_entrada_salida: Atributo condicional para registrar la vía de ingreso o salida de los bienes y/o mercancías en territorio nacional.
    :param total_dist_rec: Atributo condicional para indicar en kilómetros, la suma de las distancias recorridas, registradas en el atributo “DistanciaRecorrida”, para el traslado de los bienes y/o mercancías.
    :param figura_transporte: Nodo condicional para indicar los datos de la(s) figura(s) del transporte que interviene(n) en el traslado de los bienes y/o mercancías realizado a través de los distintos medios de transporte dentro del territorio nacional, cuando el dueño de dicho medio sea diferente del emisor del comprobante con el complemento Carta Porte.
    """
    
    tag = '{http://www.sat.gob.mx/CartaPorte20}CartaPorte'
    version = '2.0'
    
    def __init__(
            self,
            transp_internac: str,
            ubicaciones: Ubicacion | dict | Sequence[Ubicacion | dict],
            mercancias: Mercancias | dict,
            entrada_salida_merc: str = None,
            pais_origen_destino: str = None,
            via_entrada_salida: str = None,
            total_dist_rec: Decimal | int = None,
            figura_transporte: TiposFigura | dict | Sequence[TiposFigura | dict] = None,
    ): 
        super().__init__({
            'Version': self.version,
            'TranspInternac': transp_internac,
            'Ubicaciones': ubicaciones,
            'Mercancias': mercancias,
            'EntradaSalidaMerc': entrada_salida_merc,
            'PaisOrigenDestino': pais_origen_destino,
            'ViaEntradaSalida': via_entrada_salida,
            'TotalDistRec': total_dist_rec,
            'FiguraTransporte': figura_transporte,
        })
        




import logging

from .models import Payment, PaymentsDetails, SatCFDI
from .. import iterate

logger = logging.getLogger(__name__)


def format_head(cfdi):
    return str(cfdi["TipoDeComprobante"]) + " " + cfdi["Version"] + "\n" + \
           str(cfdi.uuid) + "\n- " + cfdi.name


def format_head_pre(cfdi):
    return str(cfdi["TipoDeComprobante"]) + " " + cfdi["Version"] + "\n" + \
           "*" + cfdi["Serie"] + cfdi["Folio"]


def format_fecha(cfdi):
    return str(cfdi["Fecha"]) + "\n" + cfdi["LugarExpedicion"].code + "\n" + str(cfdi["Receptor"]["UsoCFDI"])


def format_emisor(cfdi):
    emisor = cfdi["Emisor"]
    return emisor["Nombre"] + "\n" + emisor["Rfc"] + "\n" + str(emisor["RegimenFiscal"])


def format_receptor(cfdi):
    receptor = cfdi["Receptor"]
    return receptor.get("Nombre", "") + "\n" + receptor["Rfc"] + " - " + receptor.get("DomicilioFiscalReceptor", "") + "\n" + str(receptor.get("RegimenFiscalReceptor", ""))


def format_forma_pago(cfdi):
    return str(cfdi.get("FormaPago")) + "\n" + \
           str(cfdi.get("MetodoPago")) + "\n" + \
           str(cfdi["Moneda"])


def format_forma_pago_dr(payment: Payment):
    if payment.pago is None:
        return format_forma_pago(payment.comprobante)

    return str(payment.pago["FormaDePagoP"]) + "\n" + \
           str(payment.docto_relacionado.get("MetodoDePagoDR")) + "\n" + \
           str(payment.docto_relacionado["MonedaDR"])


def format_fecha_pago(payment: PaymentsDetails):
    if payment.pago:
        return payment.pago["FechaPago"]

    def_pagado = payment.comprobante.saldo_pendiente == 0
    if def_pagado:
        return payment.comprobante["Fecha"]
    else:
        return "Pendiente"


def format_estado_cfdi(cfdi: SatCFDI):
    estatus = cfdi.consulta_estado()
    if not estatus:
        return ""

    return (
            (estatus["Estado"] or "") + "\n" +
            (estatus["EsCancelable"] or "") + "\n" +
            (estatus["EstatusCancelacion"] or "")
    )


def format_pagos(cfdi: SatCFDI):
    response = []

    if cfdi["TipoDeComprobante"] == "P":
        for pago in cfdi["Complemento"]["Pagos"]["Pago"]:
            for doc_rel in pago.get("DoctoRelacionado", []):
                response.append(doc_rel["IdDocumento"])
                response.append('- ' + doc_rel.get("Serie", "") + doc_rel.get("Folio", ""))

    if payment_complements := cfdi.payments:
        for p in payment_complements:
            response.append(str(p.comprobante.uuid))
            response.append('- ' + p.comprobante.name)

    return "\n".join(response)


def format_relaciones(cfdi: SatCFDI):
    response = []

    for rel in iterate(cfdi.get("CfdiRelacionados")):
        response.append(str(rel["TipoRelacion"]))
        for uuid in rel["CfdiRelacionado"]:
            response.append(str(uuid))

    for c in cfdi.relations:
        response.append(str(c.cfdi_relacionados["TipoRelacion"]))
        response.append(str(c.comprobante.uuid))
        response.append(str("- " + c.comprobante.name))

    return "\n".join(response)


def format_conceptos(cfdi):
    return "\n".join(str(c["ClaveProdServ"]) + "\n- " + c['Descripcion'] for c in cfdi["Conceptos"])



"""catalogocuentas www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class Ctas(ScalarMap):
    """
    Nodo obligatorio para expresar el detalle de cada cuenta y subcuenta del catálogo.
    
    :param cod_agrup: Atributo requerido para expresar el código asociador de cuentas y subcuentas conforme al catálogo publicado en la página de internet del SAT. Se debe asociar cada cuenta y subcuenta que sea más apropiado de acuerdo con la naturaleza y preponderancia de la cuenta o subcuenta.
    :param num_cta: Atributo requerido, es la clave con que se distingue la cuenta o subcuenta en la contabilidad
    :param desc: Atributo requerido para expresar el nombre de la cuenta o subcuenta
    :param nivel: Atributo requerido para expresar el nivel en el que se encuentra la cuenta o subcuenta en el catálogo.
    :param natur: Atributo requerido para expresar la naturaleza de la cuenta o subcuenta. (D - Deudora, A - Acreedora). ( Activo = D ) ( Pasivo = A ) ( Capital = A ) ( Ingreso = A ) ( Costo = D ) ( Gasto = D ) ( Resultado Integral de Financiamiento = D y/o A ) ( Cuentas de orden = D y/o A ).
    :param sub_cta_de: Atributo opcional en el caso de subcuentas. Sirve para expresar la clave de la cuenta a la que pertenece dicha subcuenta. Se convierte en requerido cuando se cuente con la información.
    """
    
    def __init__(
            self,
            cod_agrup: str,
            num_cta: str,
            desc: str,
            nivel: int,
            natur: str,
            sub_cta_de: str = None,
    ): 
        super().__init__({
            'CodAgrup': cod_agrup,
            'NumCta': num_cta,
            'Desc': desc,
            'Nivel': nivel,
            'Natur': natur,
            'SubCtaDe': sub_cta_de,
        })
        

class Catalogo(CFDI):
    """
    Estándar de catálogo de cuentas que se entrega como parte de la contabilidad electrónica.
    
    :param rfc: Atributo requerido para expresar el RFC del contribuyente que envía los datos
    :param mes: Atributo requerido para expresar el mes en que inicia la vigencia del catálogo para la balanza
    :param anio: Atributo requerido para expresar el año en que inicia la vigencia del catálogo para la balanza
    :param ctas: Nodo obligatorio para expresar el detalle de cada cuenta y subcuenta del catálogo.
    :param sello: Atributo opcional para contener el sello digital del archivo de contabilidad electrónica. El sello deberá ser expresado cómo una cadena de texto en formato Base 64
    :param no_certificado: Atributo opcional para expresar el número de serie del certificado de sello digital que ampara el archivo de contabilidad electrónica, de acuerdo al acuse correspondiente a 20 posiciones otorgado por el sistema del SAT.
    :param certificado: Atributo opcional que sirve para expresar el certificado de sello digital que ampara al archivo de contabilidad electrónica como texto, en formato base 64.
    """
    
    tag = '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas}Catalogo'
    version = '1.1'
    
    def __init__(
            self,
            rfc: str,
            mes: str,
            anio: int,
            ctas: Ctas | dict | Sequence[Ctas | dict],
            sello: str = None,
            no_certificado: str = None,
            certificado: str = None,
    ): 
        super().__init__({
            'Version': self.version,
            'RFC': rfc,
            'Mes': mes,
            'Anio': anio,
            'Ctas': ctas,
            'Sello': sello,
            'NoCertificado': no_certificado,
            'Certificado': certificado,
        })
        




"""nomina12 http://www.sat.gob.mx/nomina12"""
from collections.abc import *
from datetime import date
from decimal import Decimal

from ... import iterate
from ... import CFDI, XElement, ScalarMap


class Incapacidad(ScalarMap):
    """
    Nodo requerido para expresar información de las incapacidades.

    :param dias_incapacidad: Atributo requerido para expresar el número de días enteros que el trabajador se incapacitó en el periodo.
    :param tipo_incapacidad: Atributo requerido para expresar la razón de la incapacidad.
    :param importe_monetario: Atributo condicional para expresar el monto del importe monetario de la incapacidad.
    """

    def __init__(
            self,
            dias_incapacidad: int,
            tipo_incapacidad: str,
            importe_monetario: Decimal | int = None,
    ):
        super().__init__({
            'DiasIncapacidad': dias_incapacidad,
            'TipoIncapacidad': tipo_incapacidad,
            'ImporteMonetario': importe_monetario,
        })


class CompensacionSaldosAFavor(ScalarMap):
    """
    Nodo condicional para expresar la información referente a la compensación de saldos a favor de un trabajador.

    :param saldo_a_favor: Atributo requerido para expresar el saldo a favor determinado por el patrón al trabajador en periodos o ejercicios anteriores.
    :param ano: Atributo requerido para expresar el año en que se determinó el saldo a favor del trabajador por el patrón que se incluye en el campo “RemanenteSalFav”.
    :param remanente_sal_fav: Atributo requerido para expresar el remanente del saldo a favor del trabajador.
    """

    def __init__(
            self,
            saldo_a_favor: Decimal | int,
            ano: int,
            remanente_sal_fav: Decimal | int,
    ):
        super().__init__({
            'SaldoAFavor': saldo_a_favor,
            'Año': ano,
            'RemanenteSalFav': remanente_sal_fav,
        })


class OtroPago(ScalarMap):
    """
    Nodo requerido para expresar la información detallada del otro pago.

    :param tipo_otro_pago: Atributo requerido para expresar la clave agrupadora bajo la cual se clasifica el otro pago.
    :param clave: Atributo requerido, representa la clave de otro pago de nómina propia de la contabilidad de cada patrón, puede conformarse desde 3 hasta 15 caracteres.
    :param concepto: Atributo requerido para la descripción del concepto de otro pago.
    :param importe: Atributo requerido para expresar el importe del concepto de otro pago.
    :param subsidio_al_empleo: Nodo condicional para expresar la información referente al subsidio al empleo del trabajador.
    :param compensacion_saldos_a_favor: Nodo condicional para expresar la información referente a la compensación de saldos a favor de un trabajador.
    """

    def __init__(
            self,
            tipo_otro_pago: str,
            clave: str,
            concepto: str,
            importe: Decimal | int,
            subsidio_al_empleo: Decimal | int = None,
            compensacion_saldos_a_favor: CompensacionSaldosAFavor | dict = None,
    ):
        super().__init__({
            'TipoOtroPago': tipo_otro_pago,
            'Clave': clave,
            'Concepto': concepto,
            'Importe': importe,
            'SubsidioAlEmpleo': subsidio_al_empleo,
            'CompensacionSaldosAFavor': compensacion_saldos_a_favor,
        })


class Deduccion(ScalarMap):
    """
    Nodo requerido para expresar la información detallada de una deducción.

    :param tipo_deduccion: Atributo requerido para registrar la clave agrupadora que clasifica la deducción.
    :param clave: Atributo requerido para la clave de deducción de nómina propia de la contabilidad de cada patrón, puede conformarse desde 3 hasta 15 caracteres.
    :param concepto: Atributo requerido para la descripción del concepto de deducción.
    :param importe: Atributo requerido para registrar el importe del concepto de deducción.
    """

    def __init__(
            self,
            tipo_deduccion: str,
            clave: str,
            concepto: str,
            importe: Decimal | int,
    ):
        super().__init__({
            'TipoDeduccion': tipo_deduccion,
            'Clave': clave,
            'Concepto': concepto,
            'Importe': importe,
        })


class Deducciones(ScalarMap):
    """
    Nodo opcional para expresar las deducciones aplicables.

    :param deduccion: Nodo requerido para expresar la información detallada de una deducción.
    """

    def __init__(
            self,
            deduccion: Deduccion | Sequence[Deduccion | dict],
    ):
        super().__init__({
            'Deduccion': deduccion,
        })


class SeparacionIndemnizacion(ScalarMap):
    """
    Nodo condicional para expresar la información detallada de otros pagos por separación.

    :param total_pagado: Atributo requerido que indica el monto total del pago.
    :param num_anos_servicio: Atributo requerido para expresar el número de años de servicio del trabajador. Se redondea al entero superior si la cifra contiene años y meses y hay más de 6 meses.
    :param ultimo_sueldo_mens_ord: Atributo requerido que indica el último sueldo mensual ordinario.
    :param ingreso_acumulable: Atributo requerido para expresar los ingresos acumulables.
    :param ingreso_no_acumulable: Atributo requerido que indica los ingresos no acumulables.
    """

    def __init__(
            self,
            total_pagado: Decimal | int,
            num_anos_servicio: int,
            ultimo_sueldo_mens_ord: Decimal | int,
            ingreso_acumulable: Decimal | int,
            ingreso_no_acumulable: Decimal | int,
    ):
        super().__init__({
            'TotalPagado': total_pagado,
            'NumAñosServicio': num_anos_servicio,
            'UltimoSueldoMensOrd': ultimo_sueldo_mens_ord,
            'IngresoAcumulable': ingreso_acumulable,
            'IngresoNoAcumulable': ingreso_no_acumulable,
        })


class JubilacionPensionRetiro(ScalarMap):
    """
    Nodo condicional para expresar la información detallada de pagos por jubilación, pensiones o haberes de retiro.

    :param ingreso_acumulable: Atributo requerido para expresar los ingresos acumulables.
    :param ingreso_no_acumulable: Atributo requerido para expresar los ingresos no acumulables.
    :param total_una_exhibicion: Atributo condicional que indica el monto total del pago cuando se realiza en una sola exhibición.
    :param total_parcialidad: Atributo condicional para expresar los ingresos totales por pago cuando se hace en parcialidades.
    :param monto_diario: Atributo condicional para expresar el monto diario percibido por jubilación, pensiones o haberes de retiro cuando se realiza en parcialidades.
    """

    def __init__(
            self,
            ingreso_acumulable: Decimal | int,
            ingreso_no_acumulable: Decimal | int,
            total_una_exhibicion: Decimal | int = None,
            total_parcialidad: Decimal | int = None,
            monto_diario: Decimal | int = None,
    ):
        super().__init__({
            'IngresoAcumulable': ingreso_acumulable,
            'IngresoNoAcumulable': ingreso_no_acumulable,
            'TotalUnaExhibicion': total_una_exhibicion,
            'TotalParcialidad': total_parcialidad,
            'MontoDiario': monto_diario,
        })


class HorasExtra(ScalarMap):
    """
    Nodo condicional para expresar las horas extra aplicables.

    :param dias: Atributo requerido para expresar el número de días en que el trabajador realizó horas extra en el periodo.
    :param tipo_horas: Atributo requerido para expresar el tipo de pago de las horas extra.
    :param horas_extra: Atributo requerido para expresar el número de horas extra trabajadas en el periodo.
    :param importe_pagado: Atributo requerido para expresar el importe pagado por las horas extra.
    """

    def __init__(
            self,
            dias: int,
            tipo_horas: str,
            horas_extra: int,
            importe_pagado: Decimal | int,
    ):
        super().__init__({
            'Dias': dias,
            'TipoHoras': tipo_horas,
            'HorasExtra': horas_extra,
            'ImportePagado': importe_pagado,
        })


class AccionesOTitulos(ScalarMap):
    """
    Nodo condicional para expresar ingresos por acciones o títulos valor que representan bienes. Se vuelve requerido cuando existan ingresos por sueldos derivados de adquisición de acciones o títulos (Art. 94, fracción VII LISR).

    :param valor_mercado: Atributo requerido para expresar el valor de mercado de las Acciones o Títulos valor al ejercer la opción.
    :param precio_al_otorgarse: Atributo requerido para expresar el precio establecido al otorgarse la opción de ingresos en acciones o títulos valor.
    """

    def __init__(
            self,
            valor_mercado: Decimal | int,
            precio_al_otorgarse: Decimal | int,
    ):
        super().__init__({
            'ValorMercado': valor_mercado,
            'PrecioAlOtorgarse': precio_al_otorgarse,
        })


class Percepcion(ScalarMap):
    """
    Nodo requerido para expresar la información detallada de una percepción

    :param tipo_percepcion: Atributo requerido para expresar la Clave agrupadora bajo la cual se clasifica la percepción.
    :param clave: Atributo requerido para expresar la clave de percepción de nómina propia de la contabilidad de cada patrón, puede conformarse desde 3 hasta 15 caracteres.
    :param concepto: Atributo requerido para la descripción del concepto de percepción
    :param importe_gravado: Atributo requerido, representa el importe gravado de un concepto de percepción.
    :param importe_exento: Atributo requerido, representa el importe exento de un concepto de percepción.
    :param acciones_o_titulos: Nodo condicional para expresar ingresos por acciones o títulos valor que representan bienes. Se vuelve requerido cuando existan ingresos por sueldos derivados de adquisición de acciones o títulos (Art. 94, fracción VII LISR).
    :param horas_extra: Nodo condicional para expresar las horas extra aplicables.
    """

    def __init__(
            self,
            tipo_percepcion: str,
            clave: str,
            concepto: str,
            importe_gravado: Decimal | int,
            importe_exento: Decimal | int,
            acciones_o_titulos: AccionesOTitulos | dict = None,
            horas_extra: HorasExtra | Sequence[HorasExtra | dict] = None,
    ):
        super().__init__({
            'TipoPercepcion': tipo_percepcion,
            'Clave': clave,
            'Concepto': concepto,
            'ImporteGravado': importe_gravado,
            'ImporteExento': importe_exento,
            'AccionesOTitulos': acciones_o_titulos,
            'HorasExtra': horas_extra,
        })


class Percepciones(ScalarMap):
    """
    Nodo condicional para expresar las percepciones aplicables.

    :param jubilacion_pension_retiro: Nodo condicional para expresar la información detallada de pagos por jubilación, pensiones o haberes de retiro.
    :param separacion_indemnizacion: Nodo condicional para expresar la información detallada de otros pagos por separación.
    """

    def __init__(
            self,
            percepcion: Percepcion | Sequence[Percepcion | dict],
            jubilacion_pension_retiro: JubilacionPensionRetiro | dict = None,
            separacion_indemnizacion: SeparacionIndemnizacion | dict = None,
    ):
        super().__init__({
            'Percepcion': percepcion,
            'JubilacionPensionRetiro': jubilacion_pension_retiro,
            'SeparacionIndemnizacion': separacion_indemnizacion,
        })


class EntidadSNCF(ScalarMap):
    """
    Nodo condicional para que las entidades adheridas al Sistema Nacional de Coordinación Fiscal realicen la identificación del origen de los recursos utilizados en el pago de nómina del personal que presta o desempeña un servicio personal subordinado en las dependencias de la entidad federativa, del municipio o demarcación territorial de la Ciudad de México, así como en sus respectivos organismos autónomos y entidades paraestatales y paramunicipales

    :param origen_recurso: Atributo requerido para identificar el origen del recurso utilizado para el pago de nómina del personal que presta o desempeña un servicio personal subordinado o asimilado a salarios en las dependencias.
    :param monto_recurso_propio: Atributo condicional para expresar el monto del recurso pagado con cargo a sus participaciones u otros ingresos locales (importe bruto de los ingresos propios, es decir total de gravados y exentos), cuando el origen es mixto.
    """

    def __init__(
            self,
            origen_recurso: str,
            monto_recurso_propio: Decimal | int = None,
    ):
        super().__init__({
            'OrigenRecurso': origen_recurso,
            'MontoRecursoPropio': monto_recurso_propio,
        })


class Emisor(ScalarMap):
    """
    Nodo condicional para expresar la información del contribuyente emisor del comprobante de nómina.

    :param curp: Atributo condicional para expresar la CURP del emisor del comprobante de nómina cuando es una persona física.
    :param registro_patronal: Atributo condicional para expresar el registro patronal, clave de ramo - pagaduría o la que le asigne la institución de seguridad social al patrón, a 20 posiciones máximo. Se debe ingresar cuando se cuente con él, o se esté obligado conforme a otras disposiciones distintas a las fiscales.
    :param rfc_patron_origen: Atributo opcional para expresar el RFC de la persona que fungió como patrón cuando el pago al trabajador se realice a través de un tercero como vehículo o herramienta de pago.
    :param entidad_sncf: Nodo condicional para que las entidades adheridas al Sistema Nacional de Coordinación Fiscal realicen la identificación del origen de los recursos utilizados en el pago de nómina del personal que presta o desempeña un servicio personal subordinado en las dependencias de la entidad federativa, del municipio o demarcación territorial de la Ciudad de México, así como en sus respectivos organismos autónomos y entidades paraestatales y paramunicipales
    """

    def __init__(
            self,
            curp: str = None,
            registro_patronal: str = None,
            rfc_patron_origen: str = None,
            entidad_sncf: EntidadSNCF | dict = None,
    ):
        super().__init__({
            'Curp': curp,
            'RegistroPatronal': registro_patronal,
            'RfcPatronOrigen': rfc_patron_origen,
            'EntidadSNCF': entidad_sncf,
        })


class SubContratacion(ScalarMap):
    """
    Nodo condicional para expresar la lista de las personas que los subcontrataron.

    :param rfc_labora: Atributo requerido para expresar el RFC de la persona que subcontrata.
    :param porcentaje_tiempo: Atributo requerido para expresar el porcentaje del tiempo que prestó sus servicios con el RFC que lo subcontrata.
    """

    def __init__(
            self,
            rfc_labora: str,
            porcentaje_tiempo: Decimal | int,
    ):
        super().__init__({
            'RfcLabora': rfc_labora,
            'PorcentajeTiempo': porcentaje_tiempo,
        })


class Receptor(ScalarMap):
    """
    Nodo requerido para precisar la información del contribuyente receptor del comprobante de nómina.

    :param curp: Atributo requerido para expresar la CURP del receptor del comprobante de nómina.
    :param tipo_contrato: Atributo requerido para expresar el tipo de contrato que tiene el trabajador.
    :param tipo_regimen: Atributo requerido para la expresión de la clave del régimen por el cual se tiene contratado al trabajador.
    :param num_empleado: Atributo requerido para expresar el número de empleado de 1 a 15 posiciones.
    :param periodicidad_pago: Atributo requerido para la forma en que se establece el pago del salario.
    :param clave_ent_fed: Atributo requerido para expresar la clave de la entidad federativa en donde el receptor del recibo prestó el servicio.
    :param num_seguridad_social: Atributo condicional para expresar el número de seguridad social del trabajador. Se debe ingresar cuando se cuente con él, o se esté obligado conforme a otras disposiciones distintas a las fiscales.
    :param fecha_inicio_rel_laboral: Atributo condicional para expresar la fecha de inicio de la relación laboral entre el empleador y el empleado. Se expresa en la forma AAAA-MM-DD, de acuerdo con la especificación ISO 8601. Se debe ingresar cuando se cuente con él, o se esté obligado conforme a otras disposiciones distintas a las fiscales.
    :param antiguedad: Atributo condicional para expresar el número de semanas o el periodo de años, meses y días que el empleado ha mantenido relación laboral con el empleador. Se debe ingresar cuando se cuente con él, o se esté obligado conforme a otras disposiciones distintas a las fiscales.
    :param sindicalizado: Atributo opcional para indicar si el trabajador está asociado a un sindicato. Si se omite se asume que no está asociado a algún sindicato.
    :param tipo_jornada: Atributo condicional para expresar el tipo de jornada que cubre el trabajador. Se debe ingresar cuando se esté obligado conforme a otras disposiciones distintas a las fiscales.
    :param departamento: Atributo opcional para la expresión del departamento o área a la que pertenece el trabajador.
    :param puesto: Atributo opcional para la expresión del puesto asignado al empleado o actividad que realiza.
    :param riesgo_puesto: Atributo opcional para expresar la clave conforme a la Clase en que deben inscribirse los patrones, de acuerdo con las actividades que desempeñan sus trabajadores, según lo previsto en el artículo 196 del Reglamento en Materia de Afiliación Clasificación de Empresas, Recaudación y Fiscalización, o conforme con la normatividad del Instituto de Seguridad Social del trabajador.  Se debe ingresar cuando se cuente con él, o se esté obligado conforme a otras disposiciones distintas a las fiscales.
    :param banco: Atributo condicional para la expresión de la clave del Banco conforme al catálogo, donde se realiza el depósito de nómina.
    :param cuenta_bancaria: Atributo condicional para la expresión de la cuenta bancaria a 11 posiciones o número de teléfono celular a 10 posiciones o número de tarjeta de crédito, débito o servicios a 15 ó 16 posiciones o la CLABE a 18 posiciones o número de monedero electrónico, donde se realiza el depósito de nómina.
    :param salario_base_cot_apor: Atributo opcional para expresar la retribución otorgada al trabajador, que se integra por los pagos hechos en efectivo por cuota diaria, gratificaciones, percepciones, alimentación, habitación, primas, comisiones, prestaciones en especie y cualquiera otra cantidad o prestación que se entregue al trabajador por su trabajo, sin considerar los conceptos que se excluyen de conformidad con el Artículo 27 de la Ley del Seguro Social, o la integración de los pagos conforme la normatividad del Instituto de Seguridad Social del trabajador. (Se emplea para pagar las cuotas y aportaciones de Seguridad Social). Se debe ingresar cuando se esté obligado conforme a otras disposiciones distintas a las fiscales.
    :param salario_diario_integrado: Atributo opcional para expresar el salario que se integra con los pagos hechos en efectivo por cuota diaria, gratificaciones, percepciones, habitación, primas, comisiones, prestaciones en especie y cualquier otra cantidad o prestación que se entregue al trabajador por su trabajo, de conformidad con el Art. 84 de la Ley Federal del Trabajo. (Se utiliza para el cálculo de las indemnizaciones). Se debe ingresar cuando se esté obligado conforme a otras disposiciones distintas a las fiscales.
    :param sub_contratacion: Nodo condicional para expresar la lista de las personas que los subcontrataron.
    """

    def __init__(
            self,
            curp: str,
            tipo_contrato: str,
            tipo_regimen: str,
            num_empleado: str,
            periodicidad_pago: str,
            clave_ent_fed: str,
            num_seguridad_social: str = None,
            fecha_inicio_rel_laboral: date = None,
            antiguedad: str = None,
            sindicalizado: str = None,
            tipo_jornada: str = None,
            departamento: str = None,
            puesto: str = None,
            riesgo_puesto: str = None,
            banco: str = None,
            cuenta_bancaria: int | str = None,
            salario_base_cot_apor: Decimal | int = None,
            salario_diario_integrado: Decimal | int = None,
            sub_contratacion: SubContratacion | Sequence[SubContratacion | dict] = None,
    ):
        super().__init__({
            'Curp': curp,
            'TipoContrato': tipo_contrato,
            'TipoRegimen': tipo_regimen,
            'NumEmpleado': num_empleado,
            'PeriodicidadPago': periodicidad_pago,
            'ClaveEntFed': clave_ent_fed,
            'NumSeguridadSocial': num_seguridad_social,
            'FechaInicioRelLaboral': fecha_inicio_rel_laboral,
            'Antigüedad': antiguedad,
            'Sindicalizado': sindicalizado,
            'TipoJornada': tipo_jornada,
            'Departamento': departamento,
            'Puesto': puesto,
            'RiesgoPuesto': riesgo_puesto,
            'Banco': banco,
            'CuentaBancaria': cuenta_bancaria,
            'SalarioBaseCotApor': salario_base_cot_apor,
            'SalarioDiarioIntegrado': salario_diario_integrado,
            'SubContratacion': sub_contratacion,
        })


# Main #
class Nomina(CFDI):
    """
    Complemento para incorporar al Comprobante Fiscal Digital por Internet (CFDI) la información que ampara conceptos de ingresos por salarios, la prestación de un servicio personal subordinado o conceptos asimilados a salarios (Nómina).

    :param tipo_nomina: Atributo requerido para indicar el tipo de nómina, puede ser O= Nómina ordinaria o E= Nómina extraordinaria.
    :param fecha_pago: Atributo requerido para la expresión de la fecha efectiva de erogación del gasto. Se expresa en la forma AAAA-MM-DD, de acuerdo con la especificación ISO 8601.
    :param fecha_inicial_pago: Atributo requerido para la expresión de la fecha inicial del período de pago. Se expresa en la forma AAAA-MM-DD, de acuerdo con la especificación ISO 8601.
    :param fecha_final_pago: Atributo requerido para la expresión de la fecha final del período de pago. Se expresa en la forma AAAA-MM-DD, de acuerdo con la especificación ISO 8601.
    :param num_dias_pagados: Atributo requerido para la expresión del número o la fracción de días pagados.
    :param receptor: Nodo requerido para precisar la información del contribuyente receptor del comprobante de nómina.
    :param emisor: Nodo condicional para expresar la información del contribuyente emisor del comprobante de nómina.
    :param percepciones: Nodo condicional para expresar las percepciones aplicables.
    :param deducciones: Nodo opcional para expresar las deducciones aplicables.
    :param otros_pagos: Nodo condicional para expresar otros pagos aplicables.
    :param incapacidades: Nodo condicional para expresar información de las incapacidades.
    :return: objeto CFDI
    """

    tag = '{http://www.sat.gob.mx/nomina12}Nomina'
    version = '1.2'

    def __init__(
            self,
            tipo_nomina: str,
            fecha_pago: date,
            fecha_inicial_pago: date,
            fecha_final_pago: date,
            num_dias_pagados: Decimal,
            receptor: Receptor | dict,
            emisor: Emisor | dict = None,
            percepciones: Percepciones | dict = None,
            deducciones: Deducciones | dict = None,
            otros_pagos: OtroPago | Sequence[OtroPago | dict] = None,
            incapacidades: Incapacidad | Sequence[Incapacidad | dict] = None,
    ):
        # total_percepciones: Atributo condicional para representar la suma de las percepciones.
        total_percepciones = None
        if percepciones:
            # TotalGravado: Atributo requerido para expresar el total de percepciones gravadas que se relacionan en el comprobante.
            percepciones['TotalGravado'] = sum(p['ImporteGravado'] for p in iterate(percepciones['Percepcion']))
            # TotalExento: Atributo requerido para expresar el total de percepciones exentas que se relacionan en el comprobante.
            percepciones['TotalExento'] = sum(p['ImporteExento'] for p in iterate(percepciones['Percepcion']))
            # TotalSueldos: Atributo condicional para expresar el total de percepciones brutas (gravadas y exentas) por sueldos y salarios y conceptos asimilados a salarios.
            percepciones['TotalSueldos'] = percepciones['TotalGravado'] + percepciones['TotalExento']

            # TotalSeparacionIndemnizacion: Atributo condicional para expresar el importe exento y gravado de las claves tipo percepción 022 Prima por Antigüedad, 023 Pagos por separación y 025 Indemnizaciones.
            deducciones['TotalSeparacionIndemnizacion'] = sum(
                p['ImporteGravado'] + p['ImporteExento'] for p in iterate(percepciones['Percepcion']) if p['TipoPercepcion'] in ['022', '023', '025'])
            # TotalJubilacionPensionRetiro: Atributo condicional para expresar el importe exento y gravado de las claves tipo percepción 039 Jubilaciones, pensiones o haberes de retiro en una exhibición y 044 Jubilaciones, pensiones o haberes de retiro en parcialidades.
            deducciones['TotalJubilacionPensionRetiro'] = sum(p['ImporteGravado'] + p['ImporteExento'] for p in iterate(percepciones['Percepcion']) if p['TipoPercepcion'] in ['039', '044'])

            total_percepciones = percepciones['TotalSueldos']

        # total_deducciones: Atributo condicional para representar la suma de las deducciones aplicables.
        total_deducciones = None
        if deducciones:
            # TotalOtrasDeducciones: Atributo condicional para expresar el total de deducciones que se relacionan en el comprobante, donde la clave de tipo de deducción sea distinta a la 002 correspondiente a ISR.
            deducciones['TotalOtrasDeducciones'] = sum(p['Importe'] for p in iterate(deducciones['Deduccion']) if p['TipoDeduccion'] != '002')
            # TotalImpuestosRetenidos: Atributo condicional para expresar el total de los impuestos federales retenidos, es decir, donde la clave de tipo de deducción sea 002 correspondiente a ISR.
            deducciones['TotalImpuestosRetenidos'] = sum(p['Importe'] for p in iterate(deducciones['Deduccion']) if p['TipoDeduccion'] == '002')

            total_deducciones = deducciones['TotalImpuestosRetenidos'] + deducciones['TotalOtrasDeducciones']

        # total_otros_pagos: Atributo condicional para representar la suma de otros pagos.
        total_otros_pagos = None
        if otros_pagos:
            total_otros_pagos = sum(p["Importe"] for p in iterate(otros_pagos))

        super().__init__({
            'Version': self.version,
            'TipoNomina': tipo_nomina,
            'FechaPago': fecha_pago,
            'FechaInicialPago': fecha_inicial_pago,
            'FechaFinalPago': fecha_final_pago,
            'NumDiasPagados': num_dias_pagados,
            'Receptor': receptor,
            'TotalPercepciones': total_percepciones,
            'TotalDeducciones': total_deducciones,
            'TotalOtrosPagos': total_otros_pagos,
            'Emisor': emisor,
            'Percepciones': percepciones,
            'Deducciones': deducciones,
            'OtrosPagos': otros_pagos,
            'Incapacidades': incapacidades,
        })



import base64

import requests
from lxml import etree
from lxml.etree import Element

from . import PAC, Accept, Document, CancelReason, CancelationAcknowledgment, Environment, TaxpayerStatus, AcceptRejectAcknowledgment
from .. import ResponseError
from .. import ScalarMap
from .. import __version__, Signer
from ..cfdi import CFDI
from ..create.cancela import cancelacionretencion, cancelacion
from ..create.cancela.aceptacionrechazo import SolicitudAceptacionRechazo
from ..transform.helpers import simple_element

parser = etree.XMLParser(remove_blank_text=True, huge_tree=True)


class RequestTransaction(ScalarMap):
    """

    :param requestor:
    :param transaction:
    :param country:
    :param entity:
    :param user:
    :param user_name:
    :param data1:
    :param data2:
    :param data3:
    """

    tag = '{http://www.fact.com.mx/schema/ws}RequestTransaction'

    def __init__(
            self,
            requestor: str = None,
            transaction: str = None,
            country: str = None,
            entity: str = None,
            user: str = None,
            user_name: str = None,
            data1: str | bytes = None,
            data2: str | bytes = None,
            data3: str | bytes = None,
    ):
        super().__init__({
            'Requestor': requestor,
            'Transaction': transaction,
            'Country': country,
            'Entity': entity,
            'User': user,
            'UserName': user_name,
            'Data1': data1,
            'Data2': data2,
            'Data3': data3,
        })


def request_transaction(data):
    self = Element('{%s}%s' % ('http://www.fact.com.mx/schema/ws', 'RequestTransaction'), nsmap={'ws': 'http://www.fact.com.mx/schema/ws'})
    el = data.get('Requestor')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}Requestor', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    el = data.get('Transaction')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}Transaction', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    el = data.get('Country')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}Country', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    el = data.get('Entity')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}Entity', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    el = data.get('User')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}User', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    el = data.get('UserName')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}UserName', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    el = data.get('Data1')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}Data1', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    el = data.get('Data2')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}Data2', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    el = data.get('Data3')
    self.append(simple_element('{http://www.fact.com.mx/schema/ws}Data3', nsmap={'ws': 'http://www.fact.com.mx/schema/ws'}, text=el))
    return self


def soap_envelope(payload):
    self = Element('{%s}%s' % ('http://schemas.xmlsoap.org/soap/envelope/', 'Envelope'), nsmap={'soapenv': 'http://schemas.xmlsoap.org/soap/envelope/'})
    self.append(etree.Element('{http://schemas.xmlsoap.org/soap/envelope/}Header', nsmap={'soapenv': 'http://schemas.xmlsoap.org/soap/envelope/'}))
    el = etree.Element('{http://schemas.xmlsoap.org/soap/envelope/}Body', nsmap={'soapenv': 'http://schemas.xmlsoap.org/soap/envelope/'})
    el.append(payload)
    self.append(el)
    return self


class MYSuite(PAC):
    """
    PAC de facturacion MYSuite

    Documentacion: https://soporte.mysuitemex.com/portal/es/kb/soporte-especializado
    """
    RFC = "MSE090205D9A"

    def __init__(self, requestor: str, country: str, user_name: str, environment=Environment.PRODUCTION):
        super().__init__(environment=environment)
        self.requestor = requestor
        self.country = country
        self.user_name = user_name

    def issue(self, cfdi: CFDI, accept: Accept = Accept.XML) -> Document:
        raise NotImplementedError()

    def stamp(self, cfdi: CFDI, accept: Accept = Accept.XML) -> Document:
        if accept & Accept.PDF:
            raise NotImplementedError("accept PDF not supported")

        rfc = cfdi["Emisor"]["Rfc"]
        xml = soap_envelope(
            request_transaction(
                data=RequestTransaction(
                    requestor=self.requestor,
                    transaction="TIMBRAR",
                    country=self.country,
                    entity=rfc,
                    user=self.requestor,
                    user_name=f"{self.country}.{rfc}.{self.user_name}",
                    data1=base64.b64encode(cfdi.xml_bytes())
                )
            )
        )
        match self.environment:
            case Environment.PRODUCTION:
                host = "https://www.mysuitecfdi.com"
            case Environment.TEST:
                host = "https://www.mysuitetest.com"
            case _:
                raise NotImplementedError("Not Supported CFDI")

        r = requests.post(
            url=f"{host}/mx.com.fact.wsfront/FactWSFront.asmx",
            headers={
                "User-Agent": __version__.__user_agent__,
                "Content-Type": "text/xml; charset=utf-8",
                "SOAPAction": "http://www.fact.com.mx/schema/ws/RequestTransaction"
            },
            data=etree.tostring(xml, xml_declaration=False, encoding="UTF-8", pretty_print=True)
        )

        if not r.ok:
            raise ResponseError(r)

        result = etree.fromstring(
            r.content,
            parser=parser
        )
        res_data_1 = result.find('{*}Body/{*}RequestTransactionResponse/{*}RequestTransactionResult/{*}ResponseData/{*}ResponseData1').text
        res_data_1 = base64.b64decode(res_data_1)

        timbre = CFDI.from_string(res_data_1)

        if cfdi.get("Complemento") is None:
            cfdi["Complemento"] = {}
        cfdi["Complemento"]["TimbreFiscalDigital"] = timbre

        return Document(
            document_id=timbre['UUID'],
            xml=cfdi.xml_bytes() if accept & Accept.XML else None,
        )

    def cancel(self, cfdi: CFDI, reason: CancelReason, substitution_id: str = None, signer: Signer = None) -> CancelationAcknowledgment:
        raise NotImplementedError()

    def recover(self, document_id: str, accept: Accept = Accept.XML) -> Document:
        raise NotImplementedError()

    def rfc_valid(self, rfc: str | list[str]) -> bool | list[bool]:
        raise NotImplementedError()

    def status(self, cfdi: CFDI) -> dict:
        raise NotImplementedError()

    def validate(self, cfdi: CFDI):
        raise NotImplementedError()

    def cancel_comprobante(self, cancelation: cancelacion.Cancelacion) -> CancelationAcknowledgment:
        raise NotImplementedError()

    def cancel_retencion(self, cancelation: cancelacionretencion.Cancelacion) -> CancelationAcknowledgment:
        raise NotImplementedError()

    def accept_reject(self, request: SolicitudAceptacionRechazo) -> AcceptRejectAcknowledgment:
        raise NotImplementedError()

    def pending(self, rfc: str) -> list[str]:
        raise NotImplementedError()

    def list_69b(self, rfc: str) -> TaxpayerStatus | None:
        raise NotImplementedError()



import base64
import random
import string
from urllib.parse import urlparse, urlunparse

from bs4 import BeautifulSoup
from requests import Response

from .. import Signer, Certificate


def generate_token(signer: Signer, code: str) -> bytes:
    rfc = signer.rfc
    num_serie = signer.certificate_number

    co = code + "|" + rfc + "|" + num_serie

    signature = signer.sign_sha1(data=co.encode())

    return base64.b64encode(
        base64.b64encode(co.encode()) + b"#" + base64.b64encode(signature.encode())
    )


def verify_token(certificate: Certificate, token: bytes) -> str:
    decoded_token = base64.b64decode(token)
    code, signature = decoded_token.split(b"#", maxsplit=2)

    code = base64.b64decode(code)
    signature = base64.b64decode(signature)

    assert certificate.verify_sha1(
        data=code,
        signature=base64.b64decode(signature)
    )

    code, rfc, num_serie = code.decode().split("|", maxsplit=3)

    assert rfc == certificate.rfc
    assert num_serie == certificate.certificate_number

    return code


def action_url(action: str | None, url: str):
    if not action:
        return url

    if action.startswith('/'):
        parts = urlparse(url)
        return urlunparse((parts.scheme, parts.netloc, action, None, None, None))

    return action


def get_form(res: Response, id=None):
    html = BeautifulSoup(res.text, 'html.parser')
    if id:
        form = html.find(id=id)
    else:
        form = html.select('form')[0]

    data = {
        i.attrs['name']: i.attrs.get('value')
        for i in form.findChildren('input')
        if 'name' in i.attrs
    }

    assert form.attrs['method'].upper() == "POST"
    return action_url(form.attrs.get('action'), res.url), data


def request_ref_headers(url):
    parts = urlparse(url)
    return {
        'origin': urlunparse((parts.scheme, parts.netloc, '', '', '', '')),
        'referer': url
    }


def request_verification_token(res: Response):
    html = BeautifulSoup(res.text, 'html.parser')
    return html.find(name='input', attrs={'name': '__RequestVerificationToken'}).attrs['value']


def random_ajax_id():
    return ''.join(random.choice(string.ascii_letters + string.digits) for _ in range(5))



import logging
import os
from decimal import Decimal
from itertools import repeat, chain
from satcfdi.create.compute import aggregate

module = 'satcfdi'
current_dir = os.path.dirname(__file__)

logging.basicConfig(level=logging.INFO)


def test_aggregate():
    vals = repeat({
        "k1": "A",
        "k2": "B",
        "v1": 10,
        "v2": 20
    }, 3)
    res = aggregate(
        vals,
        keys=("k1", 'k2'),
        values=("v1", "v2"),
    )
    assert res == [{'k1': 'A', 'k2': 'B', 'v1': 30, 'v2': 60}]

    vals = repeat({
        "k1": "A",
        "k2": "B",
        "v1": Decimal(10),
        "v2": Decimal(20)
    }, 3)
    res = aggregate(
        vals,
        keys=("k1", 'k2'),
        values=("v1", "v2"),
        project=("p1", "p2", "p3", "p4")
    )
    assert res == [{'p1': 'A', 'p2': 'B', 'p3': Decimal('30'), 'p4': Decimal('60')}]

    vals = repeat({
        "k1": "A",
        "k2": "B",
        "v1": Decimal(10),
        "v2": Decimal(20)
    }, 3)
    vals2 = repeat({
        "k1": "C",
        "k2": "B",
        "v1": Decimal(10),
        "v2": Decimal(20)
    }, 3)
    res = aggregate(
        chain(vals, vals2),
        keys=("k1", 'k2'),
        values=("v1", "v2"),
        project=("p1", "p2", "p3", "p4")
    )
    assert res == [{'p1': 'A', 'p2': 'B', 'p3': Decimal('30'), 'p4': Decimal('60')},
                   {'p1': 'C', 'p2': 'B', 'p3': Decimal('30'), 'p4': Decimal('60')}] != [{'p1': 'A', 'p2': 'B', 'p3': Decimal('30'), 'p4': Decimal('60')}]



import logging
import os

import pytest

from satcfdi.cfdi import CFDI
from satcfdi.transform import HUSO_HORARIOS
# noinspection PyUnresolvedReferences
from satcfdi.transform.catalog import CATALOGS
from satcfdi.transform.helpers import catalog_code
from tests.constants import CFDI_FILES
from tests.utils import verify_result, XElementPrettyPrinter

module = 'satcfdi'
current_dir = os.path.dirname(__file__)

logging.basicConfig(level=logging.INFO)


def verify_invoice(invoice, path):
    pp = XElementPrettyPrinter()
    verify = verify_result(data=pp.pformat(invoice), filename=f"{path}.pretty.py")
    assert verify

    # verify = verify_result(data=invoice.xml_bytes(pretty_print=True), filename=f"{path}.xml")
    # assert verify

    verify = verify_result(data=invoice.html_str(), filename=f"{path}.html")
    assert verify

    invoice.pdf_write(os.path.join(current_dir, "test_cfdi", f"{path}.pdf"))


@pytest.mark.parametrize('xml_file', CFDI_FILES)
def test_generate_pdf(caplog, xml_file):
    cfdi = CFDI.from_file(
        os.path.join(current_dir, 'cfdi_ejemplos', xml_file)
    )

    verify_invoice(cfdi, xml_file)


def test_catalog(caplog):
    code = catalog_code('{http://www.sat.gob.mx/sitio_internet/cfd/catalogos}c_FormaPago', "24")
    assert code.code == "24"
    assert code.description == 'Confusión'

    code = catalog_code('{http://www.sat.gob.mx/sitio_internet/cfd/catalogos}c_FormaPago', "-1")
    assert code.code == "-1"
    assert code.description is None
    for record in caplog.records:
        assert record.args == ('{http://www.sat.gob.mx/sitio_internet/cfd/catalogos}c_FormaPago -1',)


def test_huso_horario():
    c = CATALOGS['{http://www.sat.gob.mx/sitio_internet/cfd/catalogos}c_CodigoPostal']
    for i in c.values():
        assert i[4] in HUSO_HORARIOS





# -*- coding: utf-8 -*-
"""
HTML Converter
"""
from collections.abc import Mapping, Sequence
from html import escape as html_escape


def dumps(obj, clubbing=True, escape=True, translate_keys=None, default=str):
    """
        Convert object to HTML Table format
    """
    encoder = PY2HTMLEncoder(clubbing, escape, translate_keys, default)
    return "".join(encoder.encode(obj))


def _get_column_headers(py_input):
    """
        This method is required to implement clubbing.
        It tries to come up with column headers for your input
    """
    if py_input and isinstance(py_input, Sequence):
        if isinstance(py_input[0], Mapping):
            # use a dict to maintain consisten order
            return {k: None for r in py_input for k in r.keys()}

    return None


class PY2HTMLEncoder:
    def __init__(self, clubbing=True, escape=True, translate_keys=None, default=str):
        self.clubbing = clubbing
        self.escape = escape
        self.translate_keys = translate_keys
        self.default = default

    def encode(self, input):
        """
            Dispatch JSON input according to the outermost type and process it
            to generate the super awesome HTML format.
            We try to adhere to duck typing such that users can just pass all kinds
            of funky objects to py2html that *behave* like dicts and lists and other
            basic JSON types.
        """
        if isinstance(input, str):
            if self.escape:
                yield html_escape(input)
                return
            else:
                yield input
                return
        if isinstance(input, Mapping):
            yield from self.convert_object(input)
            return

        if isinstance(input, Sequence):
            yield from self.convert_list(input)
            return

        if self.escape:
            yield html_escape(str(self.default(input)))
            return
        else:
            yield str(self.default(input))
            return

    def convert_list(self, list_input):
        """
            Iterate over the JSON list and process it
            to generate either an HTML table or a HTML list, depending on what's inside.
            If suppose some key has array of objects and all the keys are same,
            instead of creating a new row for each such entry,
            club such values, thus it makes more sense and more readable table.

            @example:
                pyObject = {
                    "sampleData": [
                        {"a":1, "b":2, "c":3},
                        {"a":5, "b":6, "c":7}
                    ]
                }
                OUTPUT:
                _____________________________
                |               |   |   |   |
                |               | a | c | b |
                |   sampleData  |---|---|---|
                |               | 1 | 3 | 2 |
                |               | 5 | 7 | 6 |
                -----------------------------

            @contributed by: @muellermichel
        """
        if not list_input:
            return

        if self.clubbing:
            column_headers = _get_column_headers(list_input)

            if column_headers is not None:
                yield "<table>"
                yield '<thead>'
                yield '<tr><th>' + '</th><th>'.join(self.translate_keys(k) for k in column_headers) + '</th></tr>'
                yield '</thead><tbody>'
                for list_entry in list_input:
                    yield '<tr>'
                    for column_header in column_headers:
                        yield "<td>"
                        yield from self.encode(list_entry.get(column_header, ""))
                        yield '</td>'
                    yield '</tr>'
                yield '</tbody></table>'
                return

        # so you don't want or need clubbing eh? This makes @muellermichel very sad... ;(
        # alright, let's fall back to a basic list here...
        yield '<ul>'
        for child in list_input:
            yield "<li>"
            yield from self.encode(child)
            yield "</li>"
        yield '</ul>'

    def convert_object(self, dict_input):
        """
            Iterate over the JSON object and process it
            to generate the super awesome HTML Table format
        """
        if not dict_input:
            return

        yield "<table>"
        for k, v in dict_input.items():
            if v:
                yield "<tr><td class='htd'>"
                yield from self.encode(self.translate_keys(k) + ":")
                yield "</td><td>"
                yield from self.encode(v)
                yield "</td></tr>"
        yield '</table>'



"""BCE http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class Ctas(ScalarMap):
    """
    Nodo obligatorio para expresar el detalle de cada cuenta o subcuenta de la balanza de comprobación.
    
    :param num_cta: Atributo requerido para expresar la clave asignada con que se distingue la cuenta o subcuenta en el catálogo de cuentas del contribuyente.
    :param saldo_ini: Atributo requerido para expresar el monto del saldo inicial de la cuenta o subcuenta en el periodo. De acuerdo a la naturaleza de la cuenta o subcuenta, deberá de corresponder el saldo inicial, de lo contrario se entenderá que es un saldo inicial de naturaleza inversa. En caso de no existir dato, colocar cero (0)
    :param debe: Atributo requerido para expresar el monto de los movimientos deudores de la cuenta o subcuenta. En caso de no existir dato, colocar cero (0)
    :param haber: Atributo requerido para expresar el monto de los movimientos acreedores de la cuenta o subcuenta. En caso de no existir dato, colocar cero (0)
    :param saldo_fin: Atributo requerido para expresar el monto del saldo final de la cuenta o subcuenta en el periodo. De acuerdo a la naturaleza de la cuenta o subcuenta, deberá de corresponder el saldo final, de lo contrario se entenderá que es un saldo final de naturaleza inversa. En caso de no existir dato, colocar cero (0)
    """
    
    def __init__(
            self,
            num_cta: str,
            saldo_ini: Decimal | int,
            debe: Decimal | int,
            haber: Decimal | int,
            saldo_fin: Decimal | int,
    ): 
        super().__init__({
            'NumCta': num_cta,
            'SaldoIni': saldo_ini,
            'Debe': debe,
            'Haber': haber,
            'SaldoFin': saldo_fin,
        })
        

class Balanza(CFDI):
    """
    Estándar de balanza de comprobación que se entrega como parte de la contabilidad electrónica.
    
    :param rfc: Atributo requerido para expresar el RFC del contribuyente que envía los datos
    :param mes: Atributo requerido para expresar el mes al que corresponde la balanza de comprobación
    :param anio: Atributo requerido para expresar el año al que corresponde la balanza
    :param tipo_envio: Atributo requerido para expresar el tipo de envío de la balanza (N - Normal; C - Complementaria)
    :param ctas: Nodo obligatorio para expresar el detalle de cada cuenta o subcuenta de la balanza de comprobación.
    :param fecha_mod_bal: Atributo opcional para expresar la fecha de la última modificación contable de la balanza de comprobación. Es requerido cuando el tipo de Envío es complementario.
    :param sello: Atributo opcional para contener el sello digital del archivo de contabilidad electrónica. El sello deberá ser expresado cómo una cadena de texto en formato Base 64
    :param no_certificado: Atributo opcional para expresar el número de serie del certificado de sello digital que ampara el archivo de contabilidad electrónica, de acuerdo al acuse correspondiente a 20 posiciones otorgado por el sistema del SAT.
    :param certificado: Atributo opcional que sirve para expresar el certificado de sello digital que ampara al archivo de contabilidad electrónica como texto, en formato base 64.
    """
    
    tag = '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion}Balanza'
    version = '1.3'
    
    def __init__(
            self,
            rfc: str,
            mes: str,
            anio: int,
            tipo_envio: str,
            ctas: Ctas | dict | Sequence[Ctas | dict],
            fecha_mod_bal: date = None,
            sello: str = None,
            no_certificado: str = None,
            certificado: str = None,
    ): 
        super().__init__({
            'Version': self.version,
            'RFC': rfc,
            'Mes': mes,
            'Anio': anio,
            'TipoEnvio': tipo_envio,
            'Ctas': ctas,
            'FechaModBal': fecha_mod_bal,
            'Sello': sello,
            'NoCertificado': no_certificado,
            'Certificado': certificado,
        })
        




"""gceh http://www.sat.gob.mx/GastosHidrocarburos10"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class Pozos(ScalarMap):
    """
    Nodo opcional para registrar el centro de costos del Pozo al cual se encuentra relacionado el Yacimiento.
    
    :param pozo: Atributo opcional para especificar el centro de costos del pozo al cual se encuentra relacionado el costo, gasto o inversión, conforme a los “Lineamientos para la elaboración y presentación de los costos, gastos e inversiones; la procura de bienes y servicios en los contratos y asignaciones; la verificación contable y financiera de los contratos, y la actualización de regalías en contratos y del derecho de extracción de hidrocarburos”, emitidos por la Secretaria de Hacienda y Crédito Público.
    """
    
    def __init__(
            self,
            pozo: str = None,
    ): 
        super().__init__({
            'Pozo': pozo,
        })
        

class Yacimientos(ScalarMap):
    """
    Nodo opcional para registrar el centro de costos del yacimiento al cual se encuentra relacionado el campo.
    
    :param yacimiento: Atributo opcional para especificar el centro de costos del yacimiento al cual se encuentra relacionado el costo, gasto o inversión, conforme a los “Lineamientos para la elaboración y presentación de los costos, gastos e inversiones; la procura de bienes y servicios en los contratos y asignaciones; la verificación contable y financiera de los contratos, y la actualización de regalías en contratos y del derecho de extracción de hidrocarburos”, emitidos por la Secretaria de Hacienda y Crédito Público.
    :param pozos: Nodo opcional para registrar el centro de costos del Pozo al cual se encuentra relacionado el Yacimiento.
    """
    
    def __init__(
            self,
            yacimiento: str = None,
            pozos: Pozos | dict | Sequence[Pozos | dict] = None,
    ): 
        super().__init__({
            'Yacimiento': yacimiento,
            'Pozos': pozos,
        })
        

class CentroCostos(ScalarMap):
    """
    Nodo opcional para capturar los datos complementarios del centro de costos al cual se encuentra relacionado el costo, gasto o inversión, especificando el pozo, yacimiento, campo y área contractual correspondiente.
    
    :param campo: Atributo opcional para especificar el centro de costos del campo al cual se encuentra relacionado el costo, gasto o inversión, conforme a los “Lineamientos para la elaboración y presentación de los costos, gastos e inversiones; la procura de bienes y servicios en los contratos y asignaciones; la verificación contable y financiera de los contratos, y la actualización de regalías en contratos y del derecho de extracción de hidrocarburos”, emitidos por la Secretaria de Hacienda y Crédito Público.
    :param yacimientos: Nodo opcional para registrar el centro de costos del yacimiento al cual se encuentra relacionado el campo.
    """
    
    def __init__(
            self,
            campo: str = None,
            yacimientos: Yacimientos | dict | Sequence[Yacimientos | dict] = None,
    ): 
        super().__init__({
            'Campo': campo,
            'Yacimientos': yacimientos,
        })
        

class Tareas(ScalarMap):
    """
    Nodo opcional para registrar las tareas relacionadas a cada sub actividad petrolera.
    
    :param tarea_relacionada: Atributo opcional para expresar la tarea con la cual se encuentra relacionado el costo, gasto o inversión de que se trata, conforme a los “Lineamientos para la elaboración y presentación de los costos, gastos e inversiones; la procura de bienes y servicios en los contratos y asignaciones; la verificación contable y financiera de los contratos, y la actualización de regalías en contratos y del derecho de extracción de hidrocarburos”, emitidos por la Secretaria de Hacienda y Crédito Público.
    """
    
    def __init__(
            self,
            tarea_relacionada: str = None,
    ): 
        super().__init__({
            'TareaRelacionada': tarea_relacionada,
        })
        

class SubActividades(ScalarMap):
    """
    Nodo opcional para registrar las sub actividades relacionadas a cada actividad petrolera.
    
    :param sub_actividad_relacionada: Atributo opcional para expresar la subactividad con la cual se encuentra relacionado el costo, gasto o inversión de que se trata, conforme a los “Lineamientos para la elaboración y presentación de los costos, gastos e inversiones; la procura de bienes y servicios en los contratos y asignaciones; la verificación contable y financiera de los contratos, y la actualización de regalías en contratos y del derecho de extracción de hidrocarburos”, emitidos por la Secretaria de Hacienda y Crédito Público.
    :param tareas: Nodo opcional para registrar las tareas relacionadas a cada sub actividad petrolera.
    """
    
    def __init__(
            self,
            sub_actividad_relacionada: str = None,
            tareas: Tareas | dict | Sequence[Tareas | dict] = None,
    ): 
        super().__init__({
            'SubActividadRelacionada': sub_actividad_relacionada,
            'Tareas': tareas,
        })
        

class Actividades(ScalarMap):
    """
    Nodo opcional para registrar las actividades petroleras.
    
    :param actividad_relacionada: Atributo opcional para expresar la actividad con la cual se encuentra relacionado el costo, gasto o inversión de que se trata, conforme a los “Lineamientos para la elaboración y presentación de los costos, gastos e inversiones; la procura de bienes y servicios en los contratos y asignaciones; la verificación contable y financiera de los contratos, y la actualización de regalías en contratos y del derecho de extracción de hidrocarburos”, emitidos por la Secretaria de Hacienda y Crédito Público.
    :param sub_actividades: Nodo opcional para registrar las sub actividades relacionadas a cada actividad petrolera.
    """
    
    def __init__(
            self,
            actividad_relacionada: str = None,
            sub_actividades: SubActividades | dict | Sequence[SubActividades | dict] = None,
    ): 
        super().__init__({
            'ActividadRelacionada': actividad_relacionada,
            'SubActividades': sub_actividades,
        })
        

class DocumentoRelacionado(ScalarMap):
    """
    Nodo requerido para expresar la información del documento relacionado a la erogación.
    
    :param origen_erogacion: Atributo requerido para expresar el origen de la operación nacional o extranjera, por lo que cuando sea nacional, deberá señalarse el FolioFiscalVinculado y cuando sea extranjera, deberá señalarse el NumeroPedimentoVinculado.
    :param mes: Atributo requerido para expresar el mes al que corresponden los costos, gastos o inversiones efectuadas.
    :param monto_total_erogaciones: Atributo requerido para expresar el monto total del costo, gasto o inversión, según corresponda, que conste en el CFDI o en el comprobante fiscal que cumpla con lo dispuesto en la regla 2.7.1.16. (o la regla que corresponda en la RMF del ejercicio que se trate), emitido a favor del operador del consorcio, el cual se encuentra vinculado con el CFDI emitido al integrante del consorcio.
    :param folio_fiscal_vinculado: Atributo condicional para expresar el folio fiscal del CFDI emitido al operador del consorcio con motivo de los costos, gastos o inversiones.
    :param rfc_proveedor: Atributo condicional para expresar el RFC del proveedor que expidió el CFDI a favor del operador del consorcio, con el que se encuentra vinculado el costo, gasto o inversión.
    :param monto_total_iva: Atributo condicional para expresar el monto total del IVA del CFDI expedido al operador del consorcio, el cual se encuentra vinculado con el CFDI emitido al integrante del consorcio.
    :param monto_retencion_isr: Atributo condicional para expresar el monto de retención del ISR que conste en el CFDI expedido al operador del consorcio.
    :param monto_retencion_iva: Atributo condicional para expresar el monto de retención del IVA que conste en el CFDI expedido al operador del consorcio.
    :param monto_retencion_otros_impuestos: Atributo condicional para expresar el monto de retención de otros impuestos que conste en el CFDI expedido al operador del consorcio.
    :param numero_pedimento_vinculado: Atributo condicional para expresar el número del pedimento de importación tramitado por el operador del consorcio con motivo de los costos, gastos o inversiones realizados en el extranjero, se expresa en el siguiente formato: últimos 2 dígitos del año de validación seguidos por dos espacios, 2 dígitos de la aduana de despacho seguidos por dos espacios, 4 dígitos del número de la patente seguidos por dos espacios, 1 dígito que corresponde al último dígito del año en curso, salvo que se trate de un pedimento consolidado iniciado en el año inmediato anterior o del pedimento original de una rectificación, seguido de 6 dígitos de la numeración progresiva por aduana.
    :param clave_pedimento_vinculado: Atributo condicional para expresar la clave del pedimento de importación tramitado por el operador del consorcio con motivo de los costos, gastos o inversiones, realizadas en el extranjero.
    :param clave_pago_pedimento_vinculado: Atributo condicional para expresar la clave de pago del pedimento tramitado por el operador del consorcio con motivo de los costos, gastos o inversiones, realizadas en el extranjero.
    :param monto_iva_pedimento: Atributo condicional para expresar el IVA pagado del pedimento de importación tramitado por el operador del consorcio con motivo de los costos, gastos o inversiones, realizadas en el extranjero.
    :param otros_impuestos_pagados_pedimento: Atributo condicional para expresar el monto total de otros impuestos pagados en el pedimento de importación, diferente de IVA, tramitado por el operador del consorcio con motivo de los costos, gastos o inversiones, realizadas en el extranjero.
    :param fecha_folio_fiscal_vinculado: Atributo condicional para expresar la fecha del comprobante fiscal, emitido al operador del consorcio con motivo de los costos, gastos o inversiones. Se expresa en la forma aaaa-mm-dd, de acuerdo con la especificación ISO 8601.
    """
    
    def __init__(
            self,
            origen_erogacion: str,
            mes: str,
            monto_total_erogaciones: Decimal | int,
            folio_fiscal_vinculado: str = None,
            rfc_proveedor: str = None,
            monto_total_iva: Decimal | int = None,
            monto_retencion_isr: Decimal | int = None,
            monto_retencion_iva: Decimal | int = None,
            monto_retencion_otros_impuestos: Decimal | int = None,
            numero_pedimento_vinculado: str = None,
            clave_pedimento_vinculado: str = None,
            clave_pago_pedimento_vinculado: str = None,
            monto_iva_pedimento: Decimal | int = None,
            otros_impuestos_pagados_pedimento: Decimal | int = None,
            fecha_folio_fiscal_vinculado: date = None,
    ): 
        super().__init__({
            'OrigenErogacion': origen_erogacion,
            'Mes': mes,
            'MontoTotalErogaciones': monto_total_erogaciones,
            'FolioFiscalVinculado': folio_fiscal_vinculado,
            'RFCProveedor': rfc_proveedor,
            'MontoTotalIVA': monto_total_iva,
            'MontoRetencionISR': monto_retencion_isr,
            'MontoRetencionIVA': monto_retencion_iva,
            'MontoRetencionOtrosImpuestos': monto_retencion_otros_impuestos,
            'NumeroPedimentoVinculado': numero_pedimento_vinculado,
            'ClavePedimentoVinculado': clave_pedimento_vinculado,
            'ClavePagoPedimentoVinculado': clave_pago_pedimento_vinculado,
            'MontoIVAPedimento': monto_iva_pedimento,
            'OtrosImpuestosPagadosPedimento': otros_impuestos_pagados_pedimento,
            'FechaFolioFiscalVinculado': fecha_folio_fiscal_vinculado,
        })
        

class Erogacion(ScalarMap):
    """
    Nodo requerido para capturar los datos de la erogación.
    
    :param tipo_erogacion: Atributo requerido para señalar el tipo de erogación realizada por el operador.
    :param montocu_erogacion: Atributo requerido para expresar el importe de cada uno de los costos, gastos o inversiones efectuados en el mes de que se trate y que integran el total del monto del CFDI emitido al integrante del consorcio y que se encuentran amparados en el CFDI o en el comprobante fiscal que cumpla con lo dispuesto en la regla 2.7.1.16. (o la regla que corresponda en la RMF del ejercicio de que se trate), expedido a favor del operador del consorcio.
    :param porcentaje: Atributo requerido para expresar el porcentaje que representa el importe total del CFDI que se expide al integrante del consorcio por los costos, gastos o inversiones efectuados en el mes de que se trate, en relación al importe total de los comprobantes expedidos al operador del consorcio.
    :param documento_relacionado: Nodo requerido para expresar la información del documento relacionado a la erogación.
    :param actividades: Nodo opcional para registrar las actividades petroleras.
    :param centro_costos: Nodo opcional para capturar los datos complementarios del centro de costos al cual se encuentra relacionado el costo, gasto o inversión, especificando el pozo, yacimiento, campo y área contractual correspondiente.
    """
    
    def __init__(
            self,
            tipo_erogacion: str,
            montocu_erogacion: Decimal | int,
            porcentaje: Decimal | int,
            documento_relacionado: DocumentoRelacionado | dict | Sequence[DocumentoRelacionado | dict],
            actividades: Actividades | dict | Sequence[Actividades | dict] = None,
            centro_costos: CentroCostos | dict | Sequence[CentroCostos | dict] = None,
    ): 
        super().__init__({
            'TipoErogacion': tipo_erogacion,
            'MontocuErogacion': montocu_erogacion,
            'Porcentaje': porcentaje,
            'DocumentoRelacionado': documento_relacionado,
            'Actividades': actividades,
            'CentroCostos': centro_costos,
        })
        

class GastosHidrocarburos(CFDI):
    """
    Complemento para incorporar la información sobre los gastos del consorcio derivados de la ejecución de un contrato de exploración o extracción de hidrocarburos.
    
    :param numero_contrato: Atributo requerido para expresar el número de contrato asignado por la Comisión Nacional de Hidrocarburos con el cual se encuentra vinculado el gasto.
    :param erogacion: Nodo requerido para capturar los datos de la erogación.
    :param area_contractual: Atributo opcional para especificar el centro de costos del área contractual al cual se encuentra relacionado el costo, gasto o inversión, conforme a los “Lineamientos para la elaboración y presentación de los costos, gastos e inversiones; la procura de bienes y servicios en los contratos y asignaciones; la verificación contable y financiera de los contratos, y la actualización de regalías en contratos y del derecho de extracción de hidrocarburos”, emitidos por la Secretaria de Hacienda y Crédito Público.
    """
    
    tag = '{http://www.sat.gob.mx/GastosHidrocarburos10}GastosHidrocarburos'
    version = '1.0'
    
    def __init__(
            self,
            numero_contrato: str,
            erogacion: Erogacion | dict | Sequence[Erogacion | dict],
            area_contractual: str = None,
    ): 
        super().__init__({
            'Version': self.version,
            'NumeroContrato': numero_contrato,
            'Erogacion': erogacion,
            'AreaContractual': area_contractual,
        })
        




from .formatters import SatCFDI
from .process import invoices_print, payments_print,  invoices_confirmation_print
from .process import invoices_export, payments_export, retentions_export, payments_retentions_export
from .process import filter_invoices_iter, filter_payments_iter, filter_retenciones_iter, complement_invoices_data
from .email import EmailManager



"""leyendasFisc http://www.sat.gob.mx/leyendasFiscales"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class Leyenda(ScalarMap):
    """
    Nodo para expresar la(s) leyenda(s) fiscal(es) que apliquen al comprobante
    
    :param texto_leyenda: Atributo requerido para especificar la leyenda fiscal
    :param disposicion_fiscal: Atributo opcional para especificar la Ley, Resolución o Disposición fiscal que regula la leyenda, deberá expresarse en siglas de mayúsculas y sin puntuación (p. ej: ISR)
    :param norma: Atributo opcional para especificar el número de Artículo o en su caso Regla que regula la obligación de la leyenda
    """
    
    def __init__(
            self,
            texto_leyenda: str,
            disposicion_fiscal: str = None,
            norma: str = None,
    ): 
        super().__init__({
            'TextoLeyenda': texto_leyenda,
            'DisposicionFiscal': disposicion_fiscal,
            'Norma': norma,
        })
        

class LeyendasFiscales(CFDI):
    """
    Nodo opcional para incluir leyendas previstas en disposiciones fiscales, distintas a las contenidas en el estándar de Comprobante Fiscal Digital (CFD) o Comprobante Fiscal Digital a través de Internet (CFDI).
    
    :param leyenda: Nodo para expresar la(s) leyenda(s) fiscal(es) que apliquen al comprobante
    """
    
    tag = '{http://www.sat.gob.mx/leyendasFiscales}LeyendasFiscales'
    version = '1.0'
    
    def __init__(
            self,
            leyenda: Leyenda | dict | Sequence[Leyenda | dict],
    ): 
        super().__init__({
            'Version': self.version,
            'Leyenda': leyenda,
        })
        




from .code import Code
from .curp import CURP
from .date_period import DatePeriod
from .rfc import RFC, RFC_Generico_Nacional, RFC_Generico_Extranjero, RFCType
from .certificate import Certificate
from .signer import Signer
from .certificate_store import CertificateStore
from . import py2html



"""operacionesconderivados http://www.sat.gob.mx/esquemas/retencionpago/1/operacionesconderivados"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class Operacionesconderivados(CFDI):
    """
    Complemento para incorporar información de las Operaciones Financieras Derivadas de Capital.
    
    :param mont_gan_acum: Atributo requerido para expresar el monto de la ganancia acumulable.
    :param mont_perd_ded: Atributo requerido para expresar el monto de la pérdida deducible.
    """
    
    tag = '{http://www.sat.gob.mx/esquemas/retencionpago/1/operacionesconderivados}Operacionesconderivados'
    version = '1.0'
    
    def __init__(
            self,
            mont_gan_acum: Decimal | int,
            mont_perd_ded: Decimal | int,
    ): 
        super().__init__({
            'Version': self.version,
            'MontGanAcum': mont_gan_acum,
            'MontPerdDed': mont_perd_ded,
        })
        




"""destruccion http://www.sat.gob.mx/certificadodestruccion"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class InformacionAduanera(ScalarMap):
    """
    Nodo opcional para expresar la información aduanera aplicable cuando se trate de un vehículo importado que se destruyó.
    
    :param num_ped_imp: Atributo requerido para expresar el número de documento aduanero que ampara la importación del vehículo a destruir.
    :param fecha: Atributo requerido para expresar la fecha de expedición del documento aduanero que ampara la importación del vehículo a destruir.
    :param aduana: Atributo requerido para precisar la aduana a través de la cual se regularizó la legal estancia en el país del vehículo destruido.
    """
    
    def __init__(
            self,
            num_ped_imp: str,
            fecha: date,
            aduana: str,
    ): 
        super().__init__({
            'NumPedImp': num_ped_imp,
            'Fecha': fecha,
            'Aduana': aduana,
        })
        

class VehiculoDestruido(ScalarMap):
    """
    Nodo requerido para expresar la información del vehículo que se destruyó.
    
    :param marca: Atributo requerido para expresar la marca del vehículo que se destruyó.
    :param tipo_o_clase: Atributo requerido para expresar el tipo o clase del vehículo que se destruyó.
    :param ano: Atributo requerido para la expresión del año del vehículo.
    :param num_placas: Atributo requerido para expresar el número de placas metálicas de identificación del servicio público federal o, en su caso, del servicio público de autotransporte de pasajero urbano o suburbano.
    :param num_fol_tarj_cir: Atributo requerido para expresar el número de folio de la tarjeta de circulación.
    :param modelo: Atributo opcional para expresar el modelo del vehículo que se destruyó.
    :param niv: Atributo opcional para expresar el número de identificación vehicular del vehículo (Cuando exista el NIV deberá incluirse este invariablemente).
    :param num_serie: Atributo opcional para expresar el número de serie de la carrocería del vehículo (en caso de contar con dicho número se deberá ingresar)
    :param num_motor: Atributo opcional para expresar el número de motor del vehículo (en caso de contar con dicho número se deberá ingresar).
    """
    
    def __init__(
            self,
            marca: str,
            tipo_o_clase: str,
            ano: int,
            num_placas: str,
            num_fol_tarj_cir: str,
            modelo: str = None,
            niv: str = None,
            num_serie: str = None,
            num_motor: str = None,
    ): 
        super().__init__({
            'Marca': marca,
            'TipooClase': tipo_o_clase,
            'Año': ano,
            'NumPlacas': num_placas,
            'NumFolTarjCir': num_fol_tarj_cir,
            'Modelo': modelo,
            'NIV': niv,
            'NumSerie': num_serie,
            'NumMotor': num_motor,
        })
        

class Certificadodedestruccion(CFDI):
    """
    Complemento para incorporar la información que integra el certificado de destrucción de vehículos destruidos por los centros de destrucción autorizados por el SAT.
    
    :param serie: Atributo requerido para expresar la serie de acuerdo al catálogo.
    :param num_fol_des_veh: Atributo requerido que expresa el número de folio para la destrucción del vehículo emitido por el Servicio de Administración Tributaria.
    :param vehiculo_destruido: Nodo requerido para expresar la información del vehículo que se destruyó.
    :param informacion_aduanera: Nodo opcional para expresar la información aduanera aplicable cuando se trate de un vehículo importado que se destruyó.
    """
    
    tag = '{http://www.sat.gob.mx/certificadodestruccion}certificadodedestruccion'
    version = '1.0'
    
    def __init__(
            self,
            serie: str,
            num_fol_des_veh: str,
            vehiculo_destruido: VehiculoDestruido | dict,
            informacion_aduanera: InformacionAduanera | dict = None,
    ): 
        super().__init__({
            'Version': self.version,
            'Serie': serie,
            'NumFolDesVeh': num_fol_des_veh,
            'VehiculoDestruido': vehiculo_destruido,
            'InformacionAduanera': informacion_aduanera,
        })
        

